#include <stdio.h>
#include <math.h>
#include <sys/time.h>
#include <X11/Xlib.h>
#include <X11/keysym.h>
#include <sys/types.h>
#include <unistd.h>
#include "cli.h"

#define CLIENTTICK (350000)
#define MINITICKLENGTH (30000)

int scn;
Display *dpy;
Window win;
GC blackgc, whitegc, flipgc, dottedgc; 
GC gamergc[MAXPLAYERS];
Pixmap backpm, icebitmap;

int win_wid, win_hgt, field_wid, field_hgt;

short majormode;
short playernum = (-1);
short gamernum = (-1);
short numgamers = 0;
long tick;
piece *pieces = NULL;
short numpieces;
double locatx = 0.0, locaty = 0.0, scale = 10.0;
short drawmode = 0;
short pieceheld;
point hand;
short cursormode;
int lastscore[MAXPLAYERS];
short icehouselist[MAXPLAYERS];
short lastgameplayers=0, lastgameyou;

double boardxmin, boardxmax, boardymin, boardymax;
double worldxmin, worldxmax, worldymin, worldymax;
zone *zones = NULL;
short numzones;

int reqoption[NUMOPTIONS], optionval[NUMOPTIONS];

siglist *sig = NULL;
short signumpieces;
point sigsize;

extern void init(), bye(), redraw_all(), titleloop();
extern void server_connect(), take_server(), read_default_signature(), run_tick();
extern void change_scale(), scroll(), redraw_field(); 
void loop(), bye(), parse_lclick(), parse_mclick(), parse_rclick();

main(argc, argv)
int argc;
char *argv[];
{
    int ix;

    if (argc<2) {
	printf("usage: %s <server>\n", argv[0]);
	exit(-1);
    };

    tick = 0;
    majormode = MAJMODE_NOTIN;
#ifndef __hpux
    srandom(getpid());
#endif /*__hpux*/

    read_default_signature();

    server_connect(argv[1], 3738);

    init_options();
    init();

    while (majormode != MAJMODE_QUIT) {
	titleloop();
	if (majormode == MAJMODE_QUIT)
	    break;
	loop();
    }
    bye();

    return 0;
}

void init_game_stuff()
{
    char buf[32];
    int ix;

    /*printf("init_game_stuff\n");*/
    for (ix=0; ix<MAXPLAYERS; ix++)
	icehouselist[ix] = 0;

    locatx = 0.0;
    locaty = 0.0;
    scale = (double)field_wid / UNIVERSE_WID;
    pieceheld = (-1);
    hand.x = 0.0;
    hand.y = 0.0;
    phands[gamernum].placing = 0;

    sprintf(buf, "Xicehouse (player %c)", gamernum+'A');
    XStoreName(dpy, win, buf);

}

void loop()
{
    XEvent event;
    char c;
    int ix, metaflag, eventp;
    fd_set readbits;
    KeySym ksym;
    long evmasks;
    int numsocks;
    struct timeval timer, tv;

    redraw_all();

    gettimeofday(&timer, (struct timezone *)NULL);

    if (ConnectionNumber(dpy) < server_socket)
	numsocks = server_socket+1;
    else
	numsocks = ConnectionNumber(dpy)+1;

    cursormode = 0;

    while (majormode == MAJMODE_IN) {

	gettimeofday(&tv, (struct timezone *)NULL);
	if ((tv.tv_sec-timer.tv_sec)*1000000 + (tv.tv_usec-timer.tv_usec) > CLIENTTICK) {
	    timer.tv_sec = tv.tv_sec;
	    timer.tv_usec = tv.tv_usec;
	    run_tick();
	}

	tv.tv_sec = 0;
	tv.tv_usec = MINITICKLENGTH;
	FD_ZERO(&readbits);
	FD_SET(ConnectionNumber(dpy), &readbits);
	FD_SET(server_socket, &readbits);
	XSync(dpy, 0);
	(void)select(numsocks, &readbits, 0, 0, &tv);
	
	if (FD_ISSET(server_socket, &readbits)) {
	    take_server();
	}

	eventp = XCheckWindowEvent(dpy, win, EVMASKS, &event);

	if (eventp) switch (event.type) {
	    case Expose:
		while (eventp && event.type==Expose) {
		    eventp = XCheckWindowEvent(dpy, win, evmasks, &event);
		}
		if (eventp)
		    XPutBackEvent(dpy, &event);
		redraw_all();
		break;
	    case ConfigureNotify:
		if (win_wid == event.xconfigure.width && win_hgt == event.xconfigure.height)
		    break;

		win_wid = event.xconfigure.width;
		win_hgt = event.xconfigure.height;
		field_wid = win_wid;
		field_hgt = win_hgt - EXTRASTUFF_HGT;

		XFreePixmap(dpy, backpm);
		backpm = XCreatePixmap(dpy, win, win_wid, win_hgt, DefaultDepth(dpy, scn));

		redraw_all();
		break;
	    case ButtonPress:
		switch (event.xbutton.button) {
		    case 1:
			parse_lclick(event.xbutton.x, event.xbutton.y);
			break;
		    case 2:
			parse_mclick(event.xbutton.x, event.xbutton.y);
			break;
		    case 3:
			cursormode = 1;
			if (pieceheld!=(-1)) {
			    double theta = pieces[pieceheld].rot;
			    XWarpPointer(dpy, None, None, 0, 0, 0, 0, (int)(100.0*cos(theta)), (int)(100.0*sin(theta)));
			}
			break;
		    default:
			break;
		}
		break;
	    case ButtonRelease:
		switch (event.xbutton.button) {
		    case 3:
			cursormode = 0;
			if (pieceheld!=(-1)) {
			    int xp, yp;
			    xp = (int)((hand.x-locatx)*scale) + (field_wid/2);
			    yp = (int)((hand.y-locaty)*scale) + (field_hgt/2);
			    XWarpPointer(dpy, None, win, 0, 0, 0, 0, xp, yp);
			}
			break;
		    default:
			break;
		}
		break;
	    case MotionNotify: {
		int xp, yp;
		point mouse;
		double theta;

		do {
		    xp = event.xmotion.x;
		    yp = event.xmotion.y;
		    eventp = XCheckWindowEvent(dpy, win, EVMASKS, &event);
		} while (eventp && event.type==MotionNotify);
		if (eventp) 
		    XPutBackEvent(dpy, &event);

		/* do something with xp, yp now */
		if (cursormode==0) {
		    mouse.x = locatx + (double)(xp - field_wid/2) / scale;
		    mouse.y = locaty + (double)(yp - field_hgt/2) / scale;
		    /* ### hand may eventually move slower than mouse */
		    hand.x = mouse.x;
		    hand.y = mouse.y;
		    if (pieceheld != (-1)) {
			pieces[pieceheld].x = hand.x;
			pieces[pieceheld].y = hand.y;
			redraw_field();
		    }
		}
		else if (cursormode==1) {
		    mouse.x = locatx + (double)(xp - field_wid/2) / scale;
		    mouse.y = locaty + (double)(yp - field_hgt/2) / scale;
		    if (mouse.y==hand.y && mouse.x==hand.x)
			theta = 0.0;
		    else
			theta = atan2(mouse.y - hand.y, mouse.x - hand.x);

		    if (pieceheld != (-1)) {
			pieces[pieceheld].rot = theta;
			redraw_field();
		    }
		}
		}
		break;
	    case KeyPress:
		ix = XLookupString(&event, &c, 1, &ksym, NULL);
		/*printf("got '%s'\n", XKeysymToString(ksym));*/
		metaflag = event.xkey.state & Mod1Mask;

		if (ix && event.xkey.y >= win_hgt-TEXTLINE_HGT) 
		    key_textline(c);
		else switch (ksym) {
		    case XK_p: {
			msg_ping msg;
			msg.type = mtype_ping;
			sendmsg(&msg);
			printf("sent ping!\n");
			}
			break;
		    case XK_q: {
			msg_askresign msg;
			msg.type = mtype_askresign;
			sendmsg(&msg);
			}
			break;
		    case XK_Q: {
			msg_crash msg;
			msg.type = mtype_crash;
			sendmsg(&msg);
			}
			break;
		    case XK_r:
			redraw_all();
			break;
		    case XK_I: {
			msg_reqicehouse msg;
			msg.type = mtype_reqicehouse;
			sendmsg(&msg);
			}
			break;
		    case XK_i: {
			set_msg1("Press shift-I to make an icehouse call.");
			}
			break;
		    case XK_Up:
			scroll(0.0, -(double)(field_hgt/16) / scale);
			redraw_field();
			break;
		    case XK_Down:
			scroll(0.0, (double)(field_hgt/16) / scale);
			redraw_field();
			break;
		    case XK_Left:
			scroll(-(double)(field_wid/16) / scale, 0.0);
			redraw_field();
			break;
		    case XK_Right:
			scroll((double)(field_wid/16) / scale, 0.0);
			redraw_field();
			break;
		    case XK_0:
			change_scale(1.25);
			break;
		    case XK_9:
			change_scale(0.8);
			break;
		    case XK_less:
			if (pieceheld != (-1)) {
			    pieces[pieceheld].rot -= 0.2;
			    redraw_field();
			}
			break;
		    case XK_greater:
			if (pieceheld != (-1)) {
			    pieces[pieceheld].rot += 0.2;
			    redraw_field();
			}
			break;
		    case XK_comma:
			if (pieceheld != (-1)) {
			    pieces[pieceheld].rot -= 0.05;
			    redraw_field();
			}
			break;
		    case XK_period:
			if (pieceheld != (-1)) {
			    pieces[pieceheld].rot += 0.05;
			    redraw_field();
			}
			break;
		    case XK_space:
			if ( (optionval[OPT_USE_PLACING_MODE]) && (pieceheld != (-1)) )  {
			    msg_sbar msg;
			    msg.type = mtype_sbar;
			    msg.piecenum = pieceheld;
			    sendmsg(&msg);

			    phands[gamernum].placing = !phands[gamernum].placing;
    			    change_scale(1.0);
			    if (phands[gamernum].placing)
				set_msg1("Now placing piece -- You may zoom in closer.");
			    else
				set_msg1("No longer placing piece.");
			}
			break;
		    case XK_M:
			drawmode++;
			if (drawmode == 5) 
			    drawmode = 0;
			redraw_field();
			break;
		    default:
			break;
		}
		break;
	    default:
		break;
	}
    }
}

void parse_lclick(glx, gly)
int glx, gly;
{
    msg_lclick msg;

    double bx = locatx + (double)(glx - field_wid/2) / scale;
    double by = locaty + (double)(gly - field_hgt/2) / scale;

    msg.type = mtype_lclick;
    msg.x = bx;
    msg.y = by;
    if (pieceheld != -1)
    {
	msg.rot = pieces[pieceheld].rot;
	msg.attack = pieces[pieceheld].attack;
    } else {
	msg.attack = -1;
    }
    sendmsg(&msg);
}

void parse_mclick(glx, gly)
int glx, gly;
{
    if (pieceheld != (-1)) {
	pieces[pieceheld].attack = !pieces[pieceheld].attack;
	redraw_field();
    }
}
