#include <stdio.h>
#include <math.h>
#include <sys/time.h>
#include <X11/Xlib.h>    
#include <X11/Xutil.h>    
#include "cli.h"
#include "ice.bm"

#define TILTCOEFF (0.00166666)
#define ATILTCOEFF (TILTCOEFF * 0.5406406757375939)

#define ZEROPIECESIZE (1.03125)
#define PIECESIZEDIFF (0.65625) 

typedef struct _bounds {
    XPoint min, max;
} bounds;

void draw_piece(), draw_zone();
extern void setup_grey(), setup_colors(); 

void init()
{
    XSetWindowAttributes attr;
    XGCValues gcvalues;
    static char dashes[2] = {1, 1};

    dpy = XOpenDisplay((char *) NULL);
    if ((Display *) NULL == dpy) {
	fprintf(stderr, "xicehouse:  could not open display.\n");
	exit(-1);
    }

    win_wid = INIT_WIN_WID;
    win_hgt = INIT_WIN_HGT;
    field_wid = win_wid;
    field_hgt = win_hgt - EXTRASTUFF_HGT;

    scn = DefaultScreen(dpy);
    win = XCreateSimpleWindow(dpy, DefaultRootWindow(dpy), 100, 100, win_wid, win_hgt, 1, BlackPixel(dpy, scn), WhitePixel(dpy, scn));

    { /* make some window managers happy */
	XWMHints wmhints;
	wmhints.flags = InputHint;
	wmhints.input = True;
	XSetWMHints(dpy, win, &wmhints);
    }

    XStoreName(dpy, win, "XIcehouse");
    attr.event_mask = EVMASKS;
    XChangeWindowAttributes(dpy, win, CWEventMask, &attr);
    XSetWindowBackground(dpy, win, BlackPixel(dpy, scn));

    XMapWindow(dpy, win);
    gcvalues.foreground = BlackPixel(dpy, scn);
    gcvalues.background = WhitePixel(dpy, scn);
    blackgc = XCreateGC(dpy, win, GCForeground|GCBackground, &gcvalues);
    XSetGraphicsExposures(dpy, blackgc, False); /* prevent leak */
    gcvalues.foreground = WhitePixel(dpy, scn);
    gcvalues.background = BlackPixel(dpy, scn);
    whitegc = XCreateGC(dpy, win, GCForeground|GCBackground, &gcvalues);
    gcvalues.line_style = LineDoubleDash;
    dottedgc = XCreateGC(dpy, win, GCForeground|GCBackground|GCLineStyle, &gcvalues);
    XSetDashes(dpy, dottedgc, 0, dashes, 2);
    gcvalues.function = GXinvert;
    flipgc = XCreateGC(dpy, win, GCFunction, &gcvalues);
    if (DefaultDepth(dpy, scn)==1)
	setup_grey(); 
    else
	setup_colors();

    icebitmap = XCreateBitmapFromData(dpy, win, ice_bits, ice_width, ice_height);

    backpm = XCreatePixmap(dpy, win, win_wid, win_hgt, DefaultDepth(dpy, scn));
    XFillRectangle(dpy, backpm, blackgc, 0, 0, win_wid, win_hgt);
}

void bye()
{
    XCloseDisplay(dpy);
}

void redraw_field()
{
    int ix;
    long bmaxx, bmaxy, bminx, bminy;
    int e1, e2;
    static bounds bnd;

    if ((optionval[OPT_JITTER]) && (majormode == MAJMODE_IN) && (((!optionval[OPT_USE_PLACING_MODE]) && (phands[gamernum].holding)) || (optionval[OPT_USE_PLACING_MODE]) && (phands[gamernum].placing)))
	jitter();

    bnd.min.x = 0;
    bnd.min.y = 0;
    bnd.max.x = field_wid;
    bnd.max.y = field_hgt;

    XFillRectangle(dpy, backpm, blackgc, 0, 0, field_wid, field_hgt);

    for (ix=0; ix<numzones; ix++)
	draw_zone(&bnd, &(zones[ix]));

    for (ix=0; ix<numpieces; ix++) 
	if (pieces[ix].heldby == (-1)) {
	    draw_piece(&bnd, pieces[ix].gamernum, pieces[ix].attack, (pieces[ix].x-locatx)*scale+(double)(field_wid/2), (pieces[ix].y-locaty)*scale+(double)(field_hgt/2), ((double)(pieces[ix].size)*PIECESIZEDIFF + ZEROPIECESIZE) * scale, pieces[ix].rot, 0);
	}
    for (ix=0; ix<numpieces; ix++) 
	if (pieces[ix].heldby != (-1)) {
	    draw_piece(&bnd, pieces[ix].gamernum, pieces[ix].attack, (pieces[ix].x-locatx)*scale+(double)(field_wid/2), (pieces[ix].y-locaty)*scale+(double)(field_hgt/2), ((double)(pieces[ix].size)*PIECESIZEDIFF + ZEROPIECESIZE) * scale, pieces[ix].rot, 1);
	}
    XCopyArea(dpy, backpm, win, blackgc, 0, 0, field_wid, field_hgt, 0, 0);
}

void redraw_all()
{
    redraw_field();
    redraw_textline();
    redraw_msgline();
    redraw_statusline();
}

void scroll(dx, dy)
double dx, dy;
{
    locatx += dx;
    locaty += dy;
    if (locatx > worldxmax)
	locatx = worldxmax;
    if (locatx < worldxmin)
	locatx = worldxmin;
    if (locaty > worldymax)
	locaty = worldymax;
    if (locaty < worldymin)
	locaty = worldymin;
}

void change_scale(dscale)
double dscale;
{
    double fac = scale;

    scale *= dscale;
    if (optionval[OPT_USE_PLACING_MODE]) {
	if ((phands[gamernum].placing) && (scale > 40.0))
	    scale = 40.0;
	if (!(phands[gamernum].placing) && (scale > 5.0))
	    scale = 5.0;
    }
    else {
	if (scale > 40.0 )
	    scale = 40.0;
    }
    if (scale < (double)field_wid / UNIVERSE_WID)
	scale = (double)field_wid / UNIVERSE_WID;
    fac = scale / fac;
    scroll((locatx-hand.x)*(1.0-fac)/fac, (locaty-hand.y)*(1.0-fac)/fac);
    redraw_field();
}

void draw_zone(bnd, zn)
bounds *bnd;
zone *zn;
{
    static XPoint pts[5];
    static bounds zbnd;
    int ix;

    zbnd.min.x = (int)((zn->boundxmin-locatx)*scale) + (field_wid/2);
    zbnd.min.y = (int)((zn->boundymin-locaty)*scale) + (field_hgt/2);
    zbnd.max.x = (int)((zn->boundxmax-locatx)*scale) + (field_wid/2);
    zbnd.max.y = (int)((zn->boundymax-locaty)*scale) + (field_hgt/2);

    if (    (zbnd.max.x >= bnd->min.x) 
	 && (zbnd.min.x < bnd->max.x) 
	 && (zbnd.max.y >= bnd->min.y) 
	 && (zbnd.min.y < bnd->max.y)) {

	for (ix=0; ix<4; ix++) {
	    pts[ix].x = (int)((zn->pts[ix].x-locatx)*scale) + (field_wid/2);
	    pts[ix].y = (int)((zn->pts[ix].y-locaty)*scale) + (field_hgt/2);
	}
	pts[4].x = pts[0].x;
	pts[4].y = pts[0].y;

	XDrawLines(dpy, backpm, whitegc, pts, 5, CoordModeOrigin);
    }
}

void draw_piece(bnd, color, attack, xp, yp, rad, rot, held)
bounds *bnd;
int color, attack;
double xp, yp, rad, rot;
int held;
{
    XPoint boundmin, boundmax;
    int ix;

    if (!attack) {
	double xoff = rad*cos(rot);
	double yoff = rad*sin(rot);
	static XSegment segs[8];
	static XPoint pts[5];

	pts[0].x = (int)(xp+xoff);
	pts[0].y = (int)(yp+yoff);
	pts[1].x = (int)(xp-yoff);
	pts[1].y = (int)(yp+xoff);
	pts[2].x = (int)(xp-xoff);
	pts[2].y = (int)(yp-yoff);
	pts[3].x = (int)(xp+yoff);
	pts[3].y = (int)(yp-xoff);
	pts[4].x = (int)(xp+TILTCOEFF*rad*(xp-(double)(field_wid/2)));
	pts[4].y = (int)(yp+TILTCOEFF*rad*(yp-(double)(field_hgt/2)));

	segs[0].x1 = pts[0].x;
	segs[0].y1 = pts[0].y;
	segs[1].x1 = pts[1].x;
	segs[1].y1 = pts[1].y;
	segs[2].x1 = pts[2].x;
	segs[2].y1 = pts[2].y;
	segs[3].x1 = pts[3].x;
	segs[3].y1 = pts[3].y;
	segs[0].x2 = pts[1].x;
	segs[0].y2 = pts[1].y;
	segs[1].x2 = pts[2].x;
	segs[1].y2 = pts[2].y;
	segs[2].x2 = pts[3].x;
	segs[2].y2 = pts[3].y;
	segs[3].x2 = pts[0].x;
	segs[3].y2 = pts[0].y;

	segs[4].x1 = pts[0].x;
	segs[4].y1 = pts[0].y;
	segs[5].x1 = pts[1].x;
	segs[5].y1 = pts[1].y;
	segs[6].x1 = pts[2].x;
	segs[6].y1 = pts[2].y;
	segs[7].x1 = pts[3].x;
	segs[7].y1 = pts[3].y;
	segs[4].x2 = pts[4].x;
	segs[4].y2 = pts[4].y;
	segs[5].x2 = pts[4].x;
	segs[5].y2 = pts[4].y;
	segs[6].x2 = pts[4].x;
	segs[6].y2 = pts[4].y;
	segs[7].x2 = pts[4].x;
	segs[7].y2 = pts[4].y;

	/* Draw only pieces in the visible area (by bounding box) */
	boundmin.x = boundmax.x = pts[0].x;
	boundmin.y = boundmax.y = pts[0].y;
	for (ix = 1; ix < 4; ix++) /* only perimeter points */
	{
	    if (pts[ix].x > boundmax.x)
		boundmax.x = pts[ix].x;
	    if (pts[ix].y > boundmax.y)
		boundmax.y = pts[ix].y;
	    if (pts[ix].x < boundmin.x)
		boundmin.x = pts[ix].x;
	    if (pts[ix].y < boundmin.y)
		boundmin.y = pts[ix].y;	
	}
	if (   (boundmax.x >= bnd->min.x) 
	    && (boundmin.x < bnd->max.x) 
	    && (boundmax.y >= bnd->min.y) 
	    && (boundmin.y < bnd->max.y))
	{
	    XFillPolygon(dpy, backpm, gamergc[color], pts, 4, Convex, CoordModeOrigin);

	    if (!held) {
		if (drawmode)
		    XDrawSegments(dpy, backpm, whitegc, segs, 8);
		else
		    XDrawSegments(dpy, backpm, whitegc, segs, 4);
	    }
	}
    }
    else { /* attack piece */
	double radtip = rad * 1.673234098593571;
	double radback = rad * 1.095412290954442;
	double radup = rad * 0.8318701146670751;
	double radatt = rad * 4.183085246483928;
	
	double xtoff = radtip*cos(rot);
	double ytoff = radtip*sin(rot);
	double xboff1 = radback*cos(rot+2.439893204463398);
	double yboff1 = radback*sin(rot+2.439893204463398);
	double xboff2 = radback*cos(rot+3.843292102716188);
	double yboff2 = radback*sin(rot+3.843292102716188);
	double xuoff1 = radup*cos(rot+2.125568145526172);
	double yuoff1 = radup*sin(rot+2.125568145526172);
	double xuoff2 = radup*cos(rot+4.157617161653414);
	double yuoff2 = radup*sin(rot+4.157617161653414);
	double xaoff = radatt*cos(rot);
	double yaoff = radatt*sin(rot);

	static XSegment segs[9];
	static XPoint pts[6];

	pts[0].x = (int)(xp+xtoff);
	pts[0].y = (int)(yp+ytoff);
	pts[1].x = (int)(xp+xboff1);
	pts[1].y = (int)(yp+yboff1);
	pts[2].x = (int)(xp+xboff2);
	pts[2].y = (int)(yp+yboff2);
	pts[3].x = (int)(xp+xuoff1+ATILTCOEFF*rad*(xp-(double)(field_wid/2)));
	pts[3].y = (int)(yp+yuoff1+ATILTCOEFF*rad*(yp-(double)(field_hgt/2)));
	pts[4].x = (int)(xp+xuoff2+ATILTCOEFF*rad*(xp-(double)(field_wid/2)));
	pts[4].y = (int)(yp+yuoff2+ATILTCOEFF*rad*(yp-(double)(field_hgt/2)));
	pts[5].x = (xp+xaoff); /* tip of attack line */
	pts[5].y = (yp+yaoff);

	segs[0].x1 = pts[0].x;
	segs[0].y1 = pts[0].y;
	segs[1].x1 = pts[1].x;
	segs[1].y1 = pts[1].y;
	segs[2].x1 = pts[2].x;
	segs[2].y1 = pts[2].y;
	segs[0].x2 = pts[1].x;
	segs[0].y2 = pts[1].y;
	segs[1].x2 = pts[2].x;
	segs[1].y2 = pts[2].y;
	segs[2].x2 = pts[0].x;
	segs[2].y2 = pts[0].y;
	segs[3].x1 = pts[3].x;
	segs[3].y1 = pts[3].y;
	segs[3].x2 = pts[4].x;
	segs[3].y2 = pts[4].y;
	segs[4].x1 = pts[0].x;
	segs[4].y1 = pts[0].y;
	segs[4].x2 = pts[3].x;
	segs[4].y2 = pts[3].y;
	segs[5].x1 = pts[0].x;
	segs[5].y1 = pts[0].y;
	segs[5].x2 = pts[4].x;
	segs[5].y2 = pts[4].y;
	segs[6].x1 = pts[1].x;
	segs[6].y1 = pts[1].y;
	segs[6].x2 = pts[3].x;
	segs[6].y2 = pts[3].y;
	segs[7].x1 = pts[2].x;
	segs[7].y1 = pts[2].y;
	segs[7].x2 = pts[4].x;
	segs[7].y2 = pts[4].y;
	/* segs[8].x1 = pts[0].x;
	segs[8].y1 = pts[0].y;
	segs[8].x2 = pts[5].x;
	segs[8].y2 = pts[5].y; */
	/* Draw only pieces in the visible area (by bounding box) */
	boundmin.x = boundmax.x = pts[0].x;
	boundmin.y = boundmax.y = pts[0].y;
	for (ix = 1; ix < 3; ix++) /* only perimeter points */
	{
	    if (pts[ix].x > boundmax.x)
		boundmax.x = pts[ix].x;
	    if (pts[ix].y > boundmax.y)
		boundmax.y = pts[ix].y;
	    if (pts[ix].x < boundmin.x)
		boundmin.x = pts[ix].x;
	    if (pts[ix].y < boundmin.y)
		boundmin.y = pts[ix].y;	
	}
	if (   (boundmax.x >= bnd->min.x) 
	    && (boundmin.x < bnd->max.x) 
	    && (boundmax.y >= bnd->min.y) 
	    && (boundmin.y < bnd->max.y))
	{
	    if (drawmode & 1) {

		XFillPolygon(dpy, backpm, gamergc[color], pts, 3, Convex, CoordModeOrigin);
		if (!held)
		    XDrawSegments(dpy, backpm, whitegc, segs, 3);
	    } 
	    else {
		XFillPolygon(dpy, backpm, gamergc[color], pts, 3, Convex, CoordModeOrigin);
		if (!held)
		    XDrawSegments(dpy, backpm, whitegc, segs, 8);

	    }
	    if (1) { /* @#@ really if (drawmode & 2) */
		XDrawLine(dpy, backpm, whitegc, pts[0].x, pts[0].y, pts[5].x, pts[5].y);
	    }
	}
    }
}

void compute_zonebounds(zn)
zone *zn;
{
    int ix;

    zn->boundxmin = zn->pts[0].x;
    zn->boundxmax = zn->pts[0].x;
    zn->boundymin = zn->pts[0].y;
    zn->boundymax = zn->pts[0].y;

    for (ix=1; ix<4; ix++) {

	if (zn->pts[ix].x < zn->boundxmin)
	    zn->boundxmin = zn->pts[ix].x;
	if (zn->pts[ix].y < zn->boundymin)
	    zn->boundymin = zn->pts[ix].y;
	if (zn->pts[ix].x > zn->boundxmax)
	    zn->boundxmax = zn->pts[ix].x;
	if (zn->pts[ix].y > zn->boundymax)
	    zn->boundymax = zn->pts[ix].y;
    }
}
