#include <stdio.h>
#include <X11/Xlib.h>
#include "cli.h"

extern void run_stick(), redraw_all(), redraw_field(), send_signature(), init_game_stuff(), compute_zonebounds();

void process_buf(ptr, msglen)
char *ptr;
int msglen;
{
    msg_generic *genptr = (msg_generic *)ptr;
    int ix;

    switch (genptr->type) {
	case mtype_generic:
	    {
	    printf("got GENERIC\n");
	    }
	    break;
	case mtype_hello: {
	    msg_hello *mptr = (msg_hello *)genptr;
	    playernum = mptr->playernum;
	    for (ix=0; ix<NUMOPTIONS; ix++)
		optionval[ix] = mptr->optvals[ix];
	    send_signature();
	    }
	    break;
	case mtype_gamefull: {
	    fprintf(stderr, "Sorry, this server is full.\n");
	    majormode = MAJMODE_QUIT;
	    }
	    break;
	case mtype_gamerunning: {
	    fprintf(stderr, "Sorry, this server has a game in progress.\n");
	    majormode = MAJMODE_QUIT;
	    }
	    break;
	case mtype_crash: {
	    fprintf(stderr, "The server has crashed.\n");
	    majormode = MAJMODE_QUIT;
	    }
	    break;
	case mtype_ping: {
	    msg_reping msg;
	    msg.type = mtype_reping;
	    sendmsg(&msg);
	    }
	    break;
	case mtype_reping: {
	    printf("server says: pong\n"); 
	    }
	    break;

	case mtype_stick: {
	    run_stick((msg_stick *)ptr);
	    }
	    break;

	case mtype_pieceplaces: {
	    msg_pieceplaces *mptr = (msg_pieceplaces *)genptr;
	    numpieces = mptr->numpieces;

	    if (pieces)
		free(pieces);
	    pieces = (piece *)malloc(sizeof(piece) * numpieces);
	    for (ix=0; ix<mptr->numpieces; ix++) {
		pieces[ix].gamernum = mptr->p[ix].gamernum;
		pieces[ix].heldby = (-1);
		pieces[ix].size = mptr->p[ix].size;
		pieces[ix].state = mptr->p[ix].state;
		pieces[ix].x = mptr->p[ix].x;
		pieces[ix].y = mptr->p[ix].y;
		pieces[ix].rot = mptr->p[ix].rot;
		pieces[ix].attack = mptr->p[ix].attack;
	    }
	    }
	    break;

	case mtype_zoneplaces: {
	    int jx;
	    msg_zoneplaces *mptr = (msg_zoneplaces *)genptr;
	    numzones = mptr->numzones;

	    if (zones) 
		free(zones);
	    zones = (zone *)malloc(sizeof(zone) * numzones);
	    for (ix=0; ix<mptr->numzones; ix++) {
		zones[ix].x = mptr->z[ix].x;
		zones[ix].y = mptr->z[ix].y;
		for (jx=0; jx<4; jx++) {
		    zones[ix].pts[jx].x = mptr->z[ix].pts[jx].x;
		    zones[ix].pts[jx].y = mptr->z[ix].pts[jx].y;
		}
		compute_zonebounds(&(zones[ix]));
	    }
	    }
	    break;

	case mtype_movepieces: {
	    msg_movepieces *mptr = (msg_movepieces *)genptr;
	    int fx;
	    for (ix=0; ix<mptr->numpieces; ix++) {
		fx = mptr->m[ix].piecenum;
		if (pieceheld == fx) {
		    cursormode = 0;
		    pieceheld = (-1);
		}
		pieces[fx].attack = mptr->m[ix].attack;
		pieces[fx].state = mptr->m[ix].state;
		pieces[fx].x = mptr->m[ix].x;
		pieces[fx].y = mptr->m[ix].y;
		pieces[fx].rot = mptr->m[ix].rot;
		pieces[fx].heldby = (-1);
	    }
	    redraw_field(); /* ### gross. Make a smaller update. */
	    }
	    break;

	case mtype_piecestatus: {
	    msg_piecestatus *mptr = (msg_piecestatus *)genptr;
	    piece *pt = &(pieces[mptr->piecenum]);
	    if (mptr->state == STATE_HELD) {
		pt->state = STATE_HELD;
		pt->heldby = mptr->gamernum;
	    }
	    else {
		pt->state = mptr->state;
		pt->heldby = (-1);
	    }
	    pt->attack = mptr->attack;
	    pt->x = mptr->x;
	    pt->y = mptr->y;
	    pt->rot = mptr->rot;
	    if (mptr->gamernum == gamernum) {
		/* it's me! */
		if (optionval[OPT_USE_PLACING_MODE] )
		    if ( (pt->state == STATE_HELD) && (pieceheld == mptr->piecenum) )
			set_msg1("You just bumped into a piece that someone else is placing");
		    else {
			if (phands[gamernum].placing)
			    set_msg1("");
			phands[gamernum].placing = 0;
			change_scale( 1.0 );
		    }
		if (pt->state == STATE_HELD) {
		    int xp, yp;
		    if ((pieceheld != mptr->piecenum) && (pieceheld != (-1))) {
			fprintf(stderr, "Ack! I'm told to grab piece %d, but I've got piece %d!\n", mptr->piecenum, pieceheld);
		    }
		    pieceheld = mptr->piecenum;
		    hand.x = pt->x;
		    hand.y = pt->y;
		    xp = (int)((hand.x-locatx)*scale) + (field_wid/2);
		    yp = (int)((hand.y-locaty)*scale) + (field_hgt/2);
		    XWarpPointer(dpy, None, win, 0, 0, 0, 0, xp, yp);
		}
		else {
		    if (pieceheld == (-1)) {
			fprintf(stderr, "Ack! I'm told to drop piece %d, but I've haven't got any pieces!\n", mptr->piecenum);
		    }
		    else if (pieceheld != mptr->piecenum) {
			fprintf(stderr, "Ack! I'm told to drop piece %d, but I've got piece %d!\n", mptr->piecenum, pieceheld);
		    }
		    cursormode = 0;
		    pieceheld = (-1);
		}
	    }
	    redraw_field(); /* ### gross. Make a smaller update. */
	    }
	    break;

	case mtype_icehouse: {
	    msg_icehouse *msg = (msg_icehouse *)ptr;
	    char buf[64];
	    static char playbuf[9] = "Player #";
	    if (msg->numiced==0) {
		/* bad call */
		if (msg->caller==gamernum) {
		    set_msg1("False call; no players are put in the icehouse.");
		    /* ### penalty */
		}
		else {
		    sprintf(buf, "Player %c made a false icehouse call.", msg->caller+'A');
		    set_msg1(buf);
		}
	    }
	    else {
		buf[0] = '\0';
		if (msg->caller==gamernum) 
		    strcat(buf, "You");
		else {
		    playbuf[7] = msg->caller+'A';
		    strcat(buf, playbuf);
		}
		strcat(buf, " put ");
		for (ix=0; ix<msg->numiced; ix++) {
		    icehouselist[msg->gamericed[ix]] = 1;
		    if (ix==msg->numiced-1 && msg->numiced>1)
			strcat(buf, "and ");
		    if (msg->gamericed[ix]==gamernum) { 
			if (gamernum==msg->caller)
			    strcat(buf, "yourself");
			else
			    strcat(buf, "you");
		    }
		    else {
			playbuf[7] = msg->gamericed[ix]+'A';
			strcat(buf, playbuf+7);
		    }
		    if (msg->numiced>2 && ix!=msg->numiced-1)
			strcat(buf, ", ");
		    else
			strcat(buf, " ");
		}   
		strcat(buf, "in the icehouse.");
		set_msg1(buf);
		redraw_statusline();
	    }
	    }
	    break;

	case mtype_changeoption: {
	    msg_changeoption *msg = (msg_changeoption *)ptr;
	    if (majormode == MAJMODE_NOTIN)
		change_option(msg->opnum, msg->newval);
	    }
	    break;

	case mtype_plerin: {
	    msg_plerin *mptr = (msg_plerin *)genptr;
	    /* pull data from plerin */
	    numgamers = mptr->numgamers;
	    gamernum = mptr->gamernum;
	    boardxmin = mptr->boardxmin;
	    boardymin = mptr->boardymin;
	    boardxmax = mptr->boardxmax;
	    boardymax = mptr->boardymax;
	    worldxmin = mptr->worldxmin;
	    worldymin = mptr->worldymin;
	    worldxmax = mptr->worldxmax;
	    worldymax = mptr->worldymax;

	    majormode = MAJMODE_IN;
	    init_game_stuff();
	    }
	    break;

	case mtype_resign: {
	    majormode = MAJMODE_NOTIN;
	    }
	    break;

	case mtype_gameover: {
	    char buf[32];
	    msg_gameover *mptr = (msg_gameover *)genptr;
	    majormode = MAJMODE_NOTIN;
	    optionval[OPT_WANTSTART] = 0;
	    reqoption[OPT_WANTSTART] = 0;
	    if (mptr->winner==(-1))
		set_msg1("The game was a tie.");
	    else if (mptr->winner==gamernum)
		set_msg1("You won! Oh happy, oh joy!");
	    else {
		sprintf(buf, "Player %c won.", mptr->winner+'A');
		set_msg1(buf);
	    }
	    for (ix=0; ix<numgamers; ix++) {
		lastscore[ix] = mptr->scores[ix];
	    }
	    lastgameplayers = numgamers;
	    lastgameyou = gamernum;
	    }
	    break;

	case mtype_piececrash: {
	    msg_piececrash *mptr = (msg_piececrash *)genptr;
	    char buf[40];
	    if (mptr->flag) {
		if (mptr->gamernum == gamernum) {
		    /* ### set some visible flag */
		    set_msg1("You crashed!");
		    if (optionval[OPT_USE_PLACING_MODE])
			phands[gamernum].placing = 0;
		    change_scale(1.0);
		}
		else {
		    sprintf(buf, "Player %c crashed!", mptr->gamernum+'A');
		    set_msg1(buf);
		}
	    }
	    else {
		if (mptr->gamernum == gamernum) {
		    /* ### unset some visible flag */
		    set_msg1("You are no longer crashed.");
		}
	    }
	    }
	    break;

	case mtype_notify: {
	    msg_notify *mptr = (msg_notify *)genptr;
	    static char buf[80];
	    switch (mptr->what) {
		case NOTIFY_OFFBOARD:
		    set_msg1("You cannot place pieces off the board.");
		    break;
		case NOTIFY_ZONEEDGE:
		    set_msg1("You cannot place pieces across an edge.");
		    break;
		case NOTIFY_FOREIGNSTASH:
		    set_msg1("You cannot place pieces on someone else's pad.");
		    break;
		case NOTIFY_NODECRASH:
		    set_msg1("You must place this piece on someone else's pad.");
		    break;
		case NOTIFY_MELTDOWN:
		    set_msg1("The first two pieces you play must be defensive.");
		    break;
		case NOTIFY_CANTDROP:
		    set_msg1("You cannot drop a piece someplace that someone is already placing a piece.");
		    break;
		case NOTIFY_CANTPLACE:
		    set_msg1("You cannot enter placing mode on top of another piece in that mode.");
		    phands[gamernum].placing = 0;
		    change_scale(1.0);
		    break;
		case NOTIFY_CANTPLACECRASHED:
		    set_msg1("You cannot enter placing while crashed.");
		    phands[gamernum].placing = 0;
		    change_scale(1.0);
		    break;
		case NOTIFY_LIFT_FOREIGNSTASH:
		    set_msg1("You cannot take pieces from someone else's pad.");
		    break;
		case NOTIFY_LIFT_DEF_BAD:
		    set_msg1("You cannot lift defensive pieces from the board.");
		    break;
		case NOTIFY_LIFT_ATT_NOTYOURS:
		    set_msg1("That piece is not attacking your piece.");
		    break;
		case NOTIFY_LIFT_ATT_OWNCOLOR:
		    set_msg1("That piece is attacking its own color.");
		    break;
		case NOTIFY_LIFT_ATT_NOTOVERICE:
		    set_msg1("That piece is not attacking an overiced piece.");
		    break;
		case NOTIFY_LIFT_ATT_NOTARGET:
		    set_msg1("That piece is not attacking anything.");
		    break;
		case NOTIFY_LIFT_ATT_TARGETATT:
		    set_msg1("That piece is attacking an attack piece.");
		    break;
		case NOTIFY_SQUANDER_NOTHING:
		    set_msg1("That attack was squandered (attacking nothing.)");
		    break;
		case NOTIFY_SQUANDER_SELF:
		    set_msg1("That attack was squandered (attacking yourself.)");
		    break;
		case NOTIFY_SQUANDER_ATTACK:
		    set_msg1("That attack was squandered (attacking an attack piece.)");
		    break;
		case NOTIFY_NOSQUANDER_NOTHING:
		    set_msg1("An attack piece may not attack nothing.");
		    break;
		case NOTIFY_NOSQUANDER_SELF:
		    set_msg1("An attack piece may not attack its own color.");
		    break;
		case NOTIFY_NOSQUANDER_ATTACK:
		    set_msg1("An attack piece may not attack another attack piece.");
		    break;
		case NOTIFY_NEED_SIG_FILE:
		    set_msg1("You must load a signature file before you start the game.");
		    set_request(OPT_WANTSTART, 0);
		    break;
		default:
		    sprintf(buf, "<unknown message, type %d>", mptr->what);
		    set_msg1(buf);
		    break;
	    }
	    }
	    break;

	default:
	    printf("other message type %d\n", genptr->type);
	    break;
    };
}

