#include <stdio.h>
#include <X11/Xlib.h>
#include "cli.h"

#define LABEL_X (100)
#define REQUEST_X (300)
#define VALUE_X (400)

#define OPTIONS_Y (150)
#define OPTION_HGT (22)

#define DRAW_STR(x, y, s) XDrawImageString(dpy, win, whitegc, (x), (y), (s), strlen(s))

static char labels[NUMOPTIONS][32] = {"Start game:", "Allow squandered attacks", "Allow variant piece lists", "Use 'placing' mode", "Cause hands to be shaky"};

void init_options()
{
    int ix;
    for (ix=0; ix<NUMOPTIONS; ix++) {
	optionval[ix] = 0;
	reqoption[ix] = 0;
    }
}

void redraw_options()
{
    int ix;
    char *val;

    DRAW_STR(LABEL_X, OPTIONS_Y, "Options:");
    DRAW_STR(REQUEST_X, OPTIONS_Y, "Your choice:");
    DRAW_STR(VALUE_X, OPTIONS_Y, "Game setting:");

    for (ix=0; ix<NUMOPTIONS; ix++) {
	DRAW_STR(LABEL_X, OPTIONS_Y+(ix+2)*OPTION_HGT, labels[ix]);
	val = reqoption[ix] ? "yes" : "no ";
	DRAW_STR(REQUEST_X, OPTIONS_Y+(ix+2)*OPTION_HGT, val);
	val = optionval[ix] ? "yes" : "no ";
	DRAW_STR(VALUE_X, OPTIONS_Y+(ix+2)*OPTION_HGT, val);
    }
}

/* set the value for a game option. Only the server can call this (via a msg_changeoption). */
void change_option(opnum, newval)
int opnum, newval;
{
    char *val;

    optionval[opnum] = newval;

    val = optionval[opnum] ? "yes" : "no ";
    DRAW_STR(VALUE_X, OPTIONS_Y+(opnum+2)*OPTION_HGT, val);
}

/* set value that client is requesting for an option, and tell the server about it. */
void set_request(opnum, newval)
int opnum, newval;
{
    char *val;
    msg_changeoption msg;

    reqoption[opnum] = newval;

    val = reqoption[opnum] ? "yes" : "no ";
    DRAW_STR(REQUEST_X, OPTIONS_Y+(opnum+2)*OPTION_HGT, val);

    msg.type = mtype_changeoption;
    msg.opnum = opnum;
    msg.newval = newval;
    sendmsg(&msg);
}

/* do click. If outside relevant area, return 0 */
int option_click(xp, yp, butnum)
int xp, yp, butnum;
{
    if (xp < LABEL_X || xp > VALUE_X+50)
	return 0;

    yp = (yp-OPTIONS_Y-8) / OPTION_HGT - 1;

    if (yp<(-2) || yp>=NUMOPTIONS)
	return 0;

    if (yp<0)
	return 1; /* in title area */

    set_request(yp, !reqoption[yp]);

    return 1;
}
