#include <stdio.h>
#ifdef SOLARIS
#include <string.h>
#define memcpy memmove
#else
#include <strings.h>
#endif
#include <errno.h>
#include <sys/param.h>
#include <pwd.h>
#include <X11/Xlib.h>    
#include "cli.h"

#define BUFFERLEN (255)

static msg_buffer mbuf;

extern char *getenv();

static char sigerrors[10][80] = {
    "",
    "Unable to open file",
    "File does not exist",
    "Bad format on first line",
    "Wrong number of pieces",
    "Bad format in body",
    "Piece coordinates out of range",
    "Bad format on second line",
    "Wrong stash pad size",
    "~ User does not exist"
};

void read_default_signature()
{
    char *homedir;
    char buf[MAXPATHLEN];
    int err;

    homedir = getenv("HOME");
    strcpy(buf, homedir);
    strcat(buf, "/.xice.sig");

    err = load_file(buf);
    if (!err) return;

    if (err!=2) /* no printed error if ~/.xice.sig doesn't exist */
	fprintf(stderr, "error in your .xice.sig file: %s\nLoading default.\n", sigerrors[err]);

    err = load_file(DEFAULTSIGFILE);
    if (!err) return;

    fprintf(stderr, "error in default .xice.sig file: %s\n", sigerrors[err]);
    exit(-1);
}

void read_alt_signature(filenm)
char *filenm;
{
    int err;
    char buf[MAXPATHLEN];

    err = load_file(filenm);
    if (!err) {
	set_msg1("Loaded.");
	return;
    }

    sprintf(buf, "error in file: %s", sigerrors[err]);
    set_msg1(buf);

    if (sig) free(sig);
    sig = NULL;
}

int load_file(fn)
char *fn;
{
    FILE *fl;
    int numsizes;
    char buf[BUFFERLEN];
    int sizelist[12];
    int ix, iy, iz, numin, attack;
    float xp, yp, rot, xsize, ysize;
    static char filenm[MAXPATHLEN];
    char id[MAXPATHLEN];
    char *x;

    if (fn[0] == '~') {
	if (fn[1] == '/') {
	    sprintf(filenm, "%s%s", getpwuid(getuid())->pw_dir, fn+1);
	}
	else {
	    for(x=fn+1;*x && *x != '/';x++) {
		id[x-(fn+1)] = *x;
	    }
	    id[x-(fn+1)] = '\0';
	    if (!getpwnam(id)) {
		return 9;
	    }
	    sprintf(filenm, "%s%s", getpwnam(id)->pw_dir, x);
	}
    }
    else
	strcpy(filenm, fn);

    fl = fopen(filenm, "r");
    if (!fl) {
	if (errno==ENOENT)
	    return 2;
	else
	    return 1;
    }

    fgets(buf, BUFFERLEN, fl);
    numsizes = sscanf(buf, "%d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d, %d", &(sizelist[0]), &(sizelist[1]), &(sizelist[2]), &(sizelist[3]), &(sizelist[4]), &(sizelist[5]), &(sizelist[6]), &(sizelist[7]), &(sizelist[8]), &(sizelist[9]), &(sizelist[10]), &(sizelist[11]));

    if (numsizes == 0) {
	return 3;
    }

    if (!optionval[OPT_ALLOW_ODD_SIGS] && 
	 (numsizes!=3 || sizelist[0]!=5 || sizelist[1]!=5 || sizelist[2]!=5)) {
	return 4;
    }

    fgets(buf, BUFFERLEN, fl);
    numin = sscanf(buf, "s %f, %f", &xsize, &ysize);
    if (numin != 2) {
	return 7;
    }

    if (xsize != 5.0 || ysize != 3.0)
	return 8;
    sigsize.x = xsize;
    sigsize.y = ysize;

    signumpieces = 0;
    for (ix=0; ix<numsizes; ix++) 
	signumpieces += sizelist[ix];

    if (sig) free(sig);
    sig = (siglist *)malloc(signumpieces * sizeof(siglist));

    iz = 0;
    for (ix=0; ix<numsizes; ix++) {
	for (iy=0; iy<sizelist[ix]; iy++) {
	    fgets(buf, BUFFERLEN, fl);
	    numin = sscanf(buf, "%f, %f, %f, %d", 
			   &xp, &yp, &rot, &attack); 
	    if (numin != 4)
		return 5;
	    sig[iz].size = ix+1;
	    sig[iz].x = xp;
	    sig[iz].y = yp;
	    sig[iz].rot = rot;
	    sig[iz].attack = attack;
	    if (sig[iz].x <= 0.0 || sig[iz].y <= 0.0 || sig[iz].x >= sigsize.x || sig[iz].y >= sigsize.y)
		return 6;

	    iz++;
	}
    }

    return 0;
}

void send_signature()
{
    msg_signature *msg = (msg_signature *)(&(mbuf));

    if (!sig) {
	/* no signature. Don't send anything. */
	return;
    }

    msg->type = mtype_signature;
    msg->numpieces = signumpieces;
    msg->x = sigsize.x;
    msg->y = sigsize.y;

    memcpy(msg->sig, sig, (int)(signumpieces*sizeof(siglist))); 

    esendmsg(msg, signumpieces*sizeof(siglist));
}
