/* textln.c -- routines to display a message line, and handle input to a filename field line */

#include <ctype.h> 
#ifdef SOLARIS
#include <string.h>
#else
#include <strings.h>
#endif
#include <X11/Xlib.h>
#include "cli.h"

#define TEXTLEN 80
#define MSGLEN 160
#define MSG2_X 224

#define INPUT_X 16
#define NEWINPUT_X 320

char input[TEXTLEN] = "";
static char newinput[TEXTLEN] = "";
static char message1[MSGLEN] = "";
static char message2[MSGLEN] = "";

void redraw_textline()
{
    XClearArea(dpy, win, 0, win_hgt-TEXTLINE_HGT, 0, 0, 0);
    XDrawLine(dpy, win, whitegc, 0, win_hgt-TEXTLINE_HGT, win_wid, win_hgt-TEXTLINE_HGT);

    XDrawImageString(dpy, win, whitegc, INPUT_X, win_hgt-TEXTLINE_HGT+16, input, strlen(input));
    if (newinput[0]) {
	XDrawImageString(dpy, win, whitegc, NEWINPUT_X, win_hgt-TEXTLINE_HGT+16, "->", 2);
	XDrawImageString(dpy, win, whitegc, NEWINPUT_X+16, win_hgt-TEXTLINE_HGT+16, newinput, strlen(newinput));
    }
}

/* add a keystroke to the input-input field. If return is hit, transfer it to the input field. */
void key_textline(ch)
char ch;
{
    int ix;

    if (isgraph(ch)) {
	ix = strlen(newinput);
	if (ix<TEXTLEN-1) {
	    newinput[ix] = ch;
	    newinput[ix+1] = '\0';
	    if (ix==0) XDrawImageString(dpy, win, whitegc, NEWINPUT_X, win_hgt-TEXTLINE_HGT+16, "->", 2);
	    XDrawImageString(dpy, win, whitegc, NEWINPUT_X+16, win_hgt-TEXTLINE_HGT+16, newinput, ix+1);
	}
    }
    else {
	switch (ch) {
	    case '\002':
	    case '\177':
		ix = strlen(newinput);
		if (ix) {
		    newinput[ix-1] = ' ';
		    XDrawImageString(dpy, win, whitegc, NEWINPUT_X+16, win_hgt-TEXTLINE_HGT+16, newinput, ix);
		    newinput[ix-1] = '\0';
		}
		break;
	    case '\013':
	    case '\001':
		newinput[0] = '\0';
		redraw_textline();
		break;
	    case '\015':
	    case '\012':
		if (newinput[0]) {
		    strcpy(input, newinput);
		    newinput[0] = '\0';
		    redraw_textline();
		}
		break;
	    default:
		break;
	}
    }
}

void redraw_msgline()
{
    XClearArea(dpy, win, 0, win_hgt-TEXTLINE_HGT-MSGLINE_HGT, 0, MSGLINE_HGT, 0);
    XDrawLine(dpy, win, whitegc, 0, win_hgt-TEXTLINE_HGT-MSGLINE_HGT, win_wid, win_hgt-TEXTLINE_HGT-MSGLINE_HGT);
    XDrawImageString(dpy, win, whitegc, 16, win_hgt-TEXTLINE_HGT-MSGLINE_HGT+16, message1, strlen(message1));
    XDrawImageString(dpy, win, whitegc, MSG2_X, win_hgt-TEXTLINE_HGT-MSGLINE_HGT+16, message2, strlen(message2));
}

/* set a message for display in the message line */
void set_msg1(newmsg)
char *newmsg;
{
    strncpy(message1, newmsg, MSGLEN);
    redraw_msgline();
}

void set_msg2(newmsg)
char *newmsg;
{
    strncpy(message2, newmsg, MSGLEN);
    redraw_msgline();
}

char *get_input_string()
{
    return input;
}

#define STATUS_BLOCK_WIDTH (64)

/* only use this during a game */
void redraw_statusline()
{
    int gx, xpos, ypos;
    static char buf[4] = "*:";
    static char buf2[4];

    ypos = win_hgt-TEXTLINE_HGT-MSGLINE_HGT-STATUSLINE_HGT;

    XClearArea(dpy, win, 0, ypos, 0, STATUSLINE_HGT, 0);
    XDrawLine(dpy, win, whitegc, 0, ypos, win_wid, ypos);

    for (gx=0; gx<numgamers; gx++) {
	xpos = STATUS_BLOCK_WIDTH * gx + 16;
	buf[0] = gx+'A';
	XDrawImageString(dpy, win, ((gx==gamernum)?blackgc:whitegc), xpos, ypos+16, buf, 2);
	XFillRectangle(dpy, win, gamergc[gx], xpos+16, ypos+4, 16, 16);
	buf2[0] = (icehouselist[gx] ? 'I' : ' ');
	XDrawImageString(dpy, win, whitegc, xpos+36, ypos+16, buf2, 1);
    }
}
