#include <stdio.h>
#include <math.h>
#ifdef SOLARIS
#include <string.h>
#else
#include <strings.h>
#endif
#include <sys/time.h>
#include <X11/Xlib.h>    
#include <X11/keysym.h>
#include <sys/types.h>
#include <signal.h>
#include "cli.h"

#define CLIENTTICK (350000)
#define MINITICKLENGTH (100000)

#define LASTGAME_Y (325)
#define LASTGAME_X (100)
#define LASTGAME_HGT (22)

#define DRAW_STR(x, y, s) XDrawImageString(dpy, win, whitegc, (x), (y), (s), strlen(s))

#define ice_width 100
#define ice_height 60

void run_titletick(), title_redraw();
extern void take_server(), redraw_options();
extern char *get_input_string();

void titleloop()
{
    XEvent event;
    char c;
    int ix, metaflag, eventp;
    KeySym ksym;
    fd_set readbits;
    long evmasks;
    int numsocks;
    struct timeval timer, tv;

    title_redraw();

    gettimeofday(&timer, (struct timezone *)NULL);

    if (ConnectionNumber(dpy) < server_socket)
	numsocks = server_socket+1;
    else
	numsocks = ConnectionNumber(dpy)+1;

    while (majormode == MAJMODE_NOTIN) {

	gettimeofday(&tv, (struct timezone *)NULL);
	if ((tv.tv_sec-timer.tv_sec)*1000000 + (tv.tv_usec-timer.tv_usec) > CLIENTTICK) {
	    timer.tv_sec = tv.tv_sec;
	    timer.tv_usec = tv.tv_usec;
	    run_titletick();
	}

	tv.tv_sec = 0;
	tv.tv_usec = MINITICKLENGTH;
	FD_ZERO(&readbits);
	FD_SET(ConnectionNumber(dpy), &readbits);
	FD_SET(server_socket, &readbits);
	XSync(dpy, 0);
	(void)select(numsocks, &readbits, 0, 0, &tv);
	
	if (FD_ISSET(server_socket, &readbits)) {
	    take_server();
	}

	evmasks = EVMASKS;
	eventp = XCheckWindowEvent(dpy, win, evmasks, &event);

	if (eventp) switch (event.type) {
	    case Expose:
		while (eventp && (event.type==Expose || event.type==NoExpose)) {
		    eventp = XCheckWindowEvent(dpy, win, evmasks, &event);
		}
		if (eventp)
		    XPutBackEvent(dpy, &event);
		title_redraw();
		break;
	    case ConfigureNotify:
		if (win_wid == event.xconfigure.width && win_hgt == event.xconfigure.height)
		    break;

		win_wid = event.xconfigure.width;
		win_hgt = event.xconfigure.height;
		field_wid = win_wid;
		field_hgt = win_hgt - EXTRASTUFF_HGT;

		XFreePixmap(dpy, backpm);
		backpm = XCreatePixmap(dpy, win, win_wid, win_hgt, DefaultDepth(dpy, scn));

		title_redraw();
		break;
	    case ButtonPress:
		if (!option_click(event.xbutton.x, event.xbutton.y, event.xbutton.button))
		    switch (event.xbutton.button) {
			case 1:
			    break;
			default:
			    break;
		    }
		break;
	    case KeyPress:
		ix = XLookupString(&event, &c, 1, &ksym, NULL);
		/*printf("got '%s'\n", XKeysymToString(ksym));*/
		metaflag = event.xkey.state & Mod1Mask;

		if (ix && event.xkey.y >= win_hgt-TEXTLINE_HGT) 
		    key_textline(c);
		else switch (ksym) {
		    case XK_q:
			majormode = MAJMODE_QUIT; /* ### maybe change */
			break;
		    case XK_Q: {
			msg_crash msg;
			msg.type = mtype_crash;
			sendmsg(&msg);
			}
			break;
		    case XK_l: {
			char *input = get_input_string();
			if (strlen(input)==0) {
			    set_msg1("Loading your default signature file.");
			    read_default_signature();
			    send_signature();
			}
			else {
			    read_alt_signature(input);
			    send_signature();
			}
			}
			break;
		    case XK_r:
			title_redraw();
			break;
		    case XK_p: {
			msg_ping msg;
			msg.type = mtype_ping;
			sendmsg(&msg);
			printf("sent ping!\n");
			}
			break;
		    case XK_s: {
			set_request(OPT_WANTSTART, !reqoption[OPT_WANTSTART]);
			}
			break;
		    default:
			break;
		}
		break;
	    case MotionNotify: {
		int xp, yp;

		/* eat all the MotionNotify events */
		do {
		    xp = event.xmotion.x;
		    yp = event.xmotion.y;
		    eventp = XCheckWindowEvent(dpy, win, EVMASKS, &event);
		} while (eventp && event.type==MotionNotify);
		if (eventp) 
		    XPutBackEvent(dpy, &event);
		/* do something with xp, yp now */
		}
		break;
	    default:
		break;
	}
    }
}

void title_numgamers_redraw()
{
    static char cbuf[8] = "0";

    sprintf(cbuf, "%d", numgamers);
    DRAW_STR(300, win_hgt-EXTRASTUFF_HGT, cbuf);
}

void title_redraw()
{
    static char buf[48];
    int ix;
    XClearWindow(dpy, win);

    XCopyPlane(dpy, icebitmap, win, blackgc, 0, 0, ice_width, ice_height, 64, 32, 1);
    DRAW_STR(200, 44, "Copyright 1992 by Andrew Plotkin (ap1i+@andrew.cmu.edu)");
    DRAW_STR(200, 64, "    and Daniel Efran (de0t+@andrew.cmu.edu)");
    DRAW_STR(200, 84, "The game of Icehouse is patented by Icehouse Games,");
    DRAW_STR(200, 104, "    PO Box 761, North College Park, MD 20740");

    redraw_textline();
    redraw_msgline();
    redraw_options();

    DRAW_STR(164, win_hgt-EXTRASTUFF_HGT, "Players connected:");
    title_numgamers_redraw();

    if (lastgameplayers) {
	DRAW_STR(LASTGAME_X, LASTGAME_Y, "Scores of last game:");
	for (ix=0; ix<lastgameplayers; ix++) {
	    sprintf(buf, "%s %c: %d %s", ((ix==lastgameyou)?"you":"   "), ix+'A', lastscore[ix], (icehouselist[ix]?"(Icehoused)":""));
	    DRAW_STR(LASTGAME_X, LASTGAME_Y+LASTGAME_HGT*(ix+1), buf);
	}
    }
}

void run_titletick()
{
    int temp, xpos, ypos;
    msg_cupdate msg;

    tick++;
}

