/* highest mtype used: 25 */

#define MAXBUF (4096)

#define MAXPLAYERS (12)

#define NUMOPTIONS (5)
#define OPT_WANTSTART (0)
#define OPT_ALLOW_SQUANDERS (1)
#define OPT_ALLOW_ODD_SIGS (2)
#define OPT_USE_PLACING_MODE (3)
#define OPT_JITTER (4)

#define UNIVERSEXMIN (-200.0)
#define UNIVERSEYMIN (-200.0)
#define UNIVERSEXMAX (200.0)
#define UNIVERSEYMAX (200.0)
#define UNIVERSE_WID (UNIVERSEXMAX-UNIVERSEXMIN)
#define UNIVERSE_HGT (UNIVERSEYMAX-UNIVERSEYMIN)

typedef struct _siglist {
    float x, y, rot;
    short size, attack;
} siglist;

typedef struct _fpoint {
    float x, y;
} fpoint;

/**** begin messages ****/

typedef struct _msg_buffer {
    int length;
    short type;
    char buf[MAXBUF];
} msg_buffer;

typedef struct _msg_generic {
    int length;
    short type;
} msg_generic;
#define mtype_generic (0)

typedef struct _msg_hello { /* (s) sent when client connects */
    int length;
    short type;
    int playernum;
    short optvals[NUMOPTIONS];
} msg_hello;
#define mtype_hello (2)

typedef struct _msg_crash { /* (c) crash server; (s) server crashed */
    int length;
    short type;
} msg_crash;
#define mtype_crash (3)

typedef struct _msg_gamefull { /* (s) game is full */
    int length;
    short type;
} msg_gamefull;
#define mtype_gamefull (10)

typedef struct _msg_gamerunning { /* (s) game is running */
    int length;
    short type;
} msg_gamerunning;
#define mtype_gamerunning (16)

typedef struct _msg_ping {
    int length;
    short type;
} msg_ping;
#define mtype_ping (4)

typedef struct _msg_reping {
    int length;
    short type;
} msg_reping;
#define mtype_reping (5)

struct msg_status_pler {
    short piecenum; 
    short attack;
    float x, y;
    float rot;
};
typedef struct _msg_stick { /* (s) server tick */
    int length;
    short type;
    short numgamers;
    struct msg_status_pler s[1]; /* list of held pieces */
} msg_stick;
#define mtype_stick (6)

struct msg_zoneplaces_sub {
    float x, y; /* must be inside zone */
    fpoint pts[4];
};
typedef struct _msg_zoneplaces { /* (s) the zones on the board */
    int length;
    short type;
    short numzones;
    struct msg_zoneplaces_sub z[1];
} msg_zoneplaces;
#define mtype_zoneplaces (18)

struct msg_pieceplaces_sub {
    short gamernum;
    short size;
    short state, attack;
    float x, y;
    float rot;
};
typedef struct _msg_pieceplaces { /* (s) the pieces and their places */
    int length;
    short type;
    short numpieces;
    struct msg_pieceplaces_sub p[1];
} msg_pieceplaces;
#define mtype_pieceplaces (11)

typedef struct _msg_plerin { /* (s) tells client to go IN; tells game setup data */
    int length;
    short type;
    short numgamers, gamernum;
    float boardxmin, boardxmax, boardymin, boardymax;
    float worldxmin, worldxmax, worldymin, worldymax;
} msg_plerin;
#define mtype_plerin (7)

typedef struct _msg_gameover { /* (s) tells client to go OUT; tells game finish data */
    int length;
    short type;
    short winner;
    short scores[MAXPLAYERS]; /* indexed by gamer */
    short piecescores[1]; /* extended */
} msg_gameover;
#define mtype_gameover (8)

typedef struct _msg_cupdate { /* (c) describe current action */
    int length;
    short type;
    short attack;
    float handx, handy;
    float rot;
} msg_cupdate;
#define mtype_cupdate (9)

typedef struct _msg_lclick { /* (c) left-click on field */
    int length;
    short type;
    float x, y, rot;
    short attack;
} msg_lclick;
#define mtype_lclick (12)

typedef struct _msg_sbar { /* (c) space-bar on field */
    int length;
    short type;
    short piecenum;
} msg_sbar;
#define mtype_sbar (26)

typedef struct _msg_piecestatus { /* (s) tell client to get/drop/etc. piece */
    int length;
    short type;
    short piecenum;
    short gamernum; /* the client that performed the action. */
    short state;
    short attack;    /* if state is not HELD */
    float x, y, rot; /* if state is not HELD */
} msg_piecestatus;
#define mtype_piecestatus (15)

struct msg_movepieces_sub {
    short piecenum;
    short state, attack;
    float x, y;
    float rot;
};
typedef struct _msg_movepieces { /* (s) tell client to move a bunch of pieces */
    int length;
    short type;
    short numpieces;
    struct msg_movepieces_sub m[1];
} msg_movepieces;
#define mtype_movepieces (25)

typedef struct _msg_askresign { /* (c) request to be NOTIN */
    int length;
    short type;
} msg_askresign;
#define mtype_askresign (13)

typedef struct _msg_resign { /* (s) tell client to go NOTIN */
    int length;
    short type;
} msg_resign;
#define mtype_resign (14)

typedef struct _msg_signature { /* (c) tell server about signature */
    int length;
    short type;
    short numpieces;
    float x, y;
    siglist sig[1];
} msg_signature;
#define mtype_signature (17)

typedef struct _msg_reqicehouse { /* (c) ask server for an Icehouse call */
    int length;
    short type;
} msg_reqicehouse;
#define mtype_reqicehouse (23)

typedef struct _msg_icehouse { /* (s) send results of an Icehouse call */
    int length;
    short type;
    short caller; /* gamer who called Icehouse */
    short numiced; /* if 0, it was a bad call */
    short gamericed[MAXPLAYERS]; /* list of gamernums who are icehoused on this call (not previously). numiced gives length of list. */
} msg_icehouse;
#define mtype_icehouse (24)

typedef struct _msg_piececrash { /* (s) tell client to go crash/nocrash */
    int length;
    short type;
    short gamernum; /* who crashed / uncrashed */
    short flag; /* 0=endcrash, 1=crash, 2=end bad icehouse call */
} msg_piececrash;
#define mtype_piececrash (19)

#define NOTIFY_FOREIGNSTASH (1)
#define NOTIFY_OFFBOARD (2)
#define NOTIFY_ZONEEDGE (3)
#define NOTIFY_LIFT_FOREIGNSTASH (4)
#define NOTIFY_LIFT_DEF_BAD (5)
#define NOTIFY_LIFT_ATT_NOTYOURS (6)
#define NOTIFY_LIFT_ATT_NOTARGET (15)
#define NOTIFY_LIFT_ATT_OWNCOLOR (16)
#define NOTIFY_LIFT_ATT_NOTOVERICE (17)
#define NOTIFY_LIFT_ATT_TARGETATT (18)
#define NOTIFY_MELTDOWN (7)
#define NOTIFY_NODECRASH (8)
#define NOTIFY_SQUANDER_NOTHING (9)
#define NOTIFY_SQUANDER_SELF (10)
#define NOTIFY_SQUANDER_ATTACK (11)
#define NOTIFY_NOSQUANDER_NOTHING (12)
#define NOTIFY_NOSQUANDER_SELF (13)
#define NOTIFY_NOSQUANDER_ATTACK (14)
#define NOTIFY_NEED_SIG_FILE (19)
#define NOTIFY_CANTPLACE (27)
#define NOTIFY_CANTDROP (28)
#define NOTIFY_CANTPLACECRASHED (29)
typedef struct _msg_notify { /* (s) tell client about something transient */
    int length;
    short type;
    short what;
} msg_notify;
#define mtype_notify (20)

typedef struct _msg_changeoption { /* (c) tell server about option change request; (s) tell client about option change */
    int length;
    short type;
    short opnum, newval;
} msg_changeoption;
#define mtype_changeoption (21)

typedef struct _msg_setoptions { /* (s) tell client new option settings */
    int length;
    short type;
    short newval[NUMOPTIONS];
} msg_setoptions;
#define mtype_setoptions (22)

