#include <stdio.h>
#include <math.h>
#include "ser.h"

pler plers[MAXPLAYERS];

static msg_buffer buf;

void emplace_piece();
extern void compute_piece();
extern double dist_to_polygon();
extern int points_attacking_piece();

/* handle an icehouse call from gamer gnum. Note rules clarification: if a player is icehoused, he cannot be icehoused again. */
void handle_icehouse(gnum)
int gnum;
{
    int gx, fx, nummov;
    short numlive, numstash;
    msg_icehouse msg;
    msg_movepieces *mptr = (msg_movepieces *)(&buf);

    msg.type = mtype_icehouse;
    msg.numiced = 0;
    msg.caller = gnum;

    nummov = 0;

    for (gx=0; gx<numgamers; gx++) {
	if (gamers[gx].icehoused)
	    continue;
	numstash = 0;
	numlive = 0;
	for (fx=0; fx<numpieces; fx++) {
	    if (pieces[fx].state == STATE_GAME
		&& pieces[fx].gamernum==gx
		&& !pieces[fx].attack
		&& points_attacking_piece(fx, -1) <= pieces[fx].size) {
		numlive++;
	    }
	    if ((pieces[fx].state == STATE_HELD && pieces[fx].heldby == gx)
		|| (pieces[fx].state == gx)) {
		numstash++;
	    }
	}
	printf("# player %d: %d live, %d on stash, limit %d\n", gx, numlive, numstash, gamers[gx].stashlimit);
	if (numstash < gamers[gx].stashlimit && numlive==0) {
	    /* gx just got icehoused */
	    gamers[gx].icehoused = 1;
	    msg.gamericed[msg.numiced] = gx;
	    msg.numiced++;
	    if (gx!=gnum)
		for (fx=0; fx<numpieces; fx++) {
		    if (pieces[fx].state == gx /* on gx's pad */
			|| (pieces[fx].state == STATE_HELD && pieces[fx].heldby == gx)) { 
			compute_piece(fx, gamers[gnum].pad.x, gamers[gnum].pad.y, pieces[fx].rot, 0);
			emplace_piece(fx, gnum);

			mptr->m[nummov].piecenum = fx;
			mptr->m[nummov].state = pieces[fx].state;
			mptr->m[nummov].attack = pieces[fx].attack;
			mptr->m[nummov].rot = pieces[fx].rot;
			mptr->m[nummov].x = pieces[fx].x;
			mptr->m[nummov].y = pieces[fx].y;
			nummov++;
		    }
		}
	}
    }

    if (msg.numiced==0) /* if nobody got icehoused */
	gamers[gnum].crashed = 2; /* impose a crash condition as a penalty */

    mptr->type = mtype_movepieces;
    mptr->numpieces = nummov;
    for (gx=0; gx<numgamers; gx++) {
	gsendmsg(gx, &msg);
	if (nummov) {
	    gesendmsg(gx, mptr, nummov*sizeof(struct msg_movepieces_sub));
	}
    }
}

void lift_piece(gnum, fnum)
int gnum, fnum;
{
    pieces[fnum].state = STATE_HELD;
    pieces[fnum].heldby = gnum;
    gamers[gnum].holding = fnum;
}

/* stash is STATE_GAME or a stash pad (gamer) number */
void emplace_piece(fnum, stash)
int fnum;
short stash;
{
    piece *pt = &(pieces[fnum]);

    pt->state = stash;

    if (pt->heldby != (-1)) {
	gamers[pt->heldby].holding = -1; /* that player isn't holding anything now */
	pt->heldby = -1; /* piece not being held by anyone now */
    }
}

/* return PLACE_OK if that's a legal place to put that piece, or PLACE_* for whatever error it is. Also return the stash pad number (or -1) of the position, in *stashnum. */
/* assumed compute_piece() has been called */
int legal_placement(gnum, piecenum, stashnum)
int gnum, piecenum;
int *stashnum;
{
    int ix, gx;
    short val;

    *stashnum = STATE_GAME;
    for (gx=0; gx<numgamers; gx++) {
	short pnum = gamers[gx].playernum;
	val = piece_pad_overlap(&(pieces[piecenum]), &(gamers[gx].pad));
	if (val==2)
	    return PLACE_ZONEEDGE; /* over edge of stash pad */
	if (val==1) {
	    /* on this stash pad. We don't bother with piece-overlap check, since you can overlap on a stash pad.*/
	    *stashnum = gx;
	    if (gnum==gx) {
		if (!gamers[gnum].crashed || numgamers==1)
		    return PLACE_STASH;
		else
		    return PLACE_NODECRASH;
	    }
	    else
		return PLACE_FOREIGNSTASH;
	}   
    }

    /* check for edges of board zone */
    val = piece_pad_overlap(&(pieces[piecenum]), &board);
    if (val==2)
	return PLACE_ZONEEDGE; /* over edge of board */
    if (val==0)
	return PLACE_OFFBOARD;

    /* check for piece-overlap with all the !held pieces */
    for (ix = 0; ix<numpieces; ix++)
    {
	if (optionval[OPT_USE_PLACING_MODE]) {
	    if (ix != piecenum && ( (pieces[ix].state == STATE_GAME) || ((pieces[ix].state == STATE_HELD) && (gamers[pieces[ix].heldby].placing)) ) )
	    {
		if (piece_overlap(&(pieces[piecenum]), &(pieces[ix])))
		    if (pieces[ix].state == STATE_HELD)
			return PLACE_CONFLICT; /* Someone is already placing something there */
		    else
			return PLACE_CRASH; /* crash */
	    }
	}
	else {
	    if ((ix != piecenum) && (pieces[ix].state != STATE_HELD))
	    {
		if (piece_overlap(&(pieces[piecenum]), &(pieces[ix])))
		    return PLACE_CRASH; /* crash */
	    }
	}
    }

    if (gamers[gnum].crashed)
	return PLACE_NODECRASH;

    if (gamers[gnum].meltstatus>0 && pieces[piecenum].attack)
	return PLACE_MELTDOWN;

    if (pieces[piecenum].attack) {
	ix = find_attacked_piece(piecenum);
	if (ix==(-1))
	    return PLACE_ATTACK_NOTHING;
	if (pieces[ix].attack)
	    return PLACE_ATTACK_ATTACK;
	if (pieces[ix].gamernum == pieces[piecenum].gamernum)
	    return PLACE_ATTACK_SELF;
    }

    return PLACE_OK;
}

void drop_piece(gnum, xp, yp, rot, att)
int gnum;
double xp, yp, rot;
short att;
{    
    msg_piecestatus msgt;
    int piecenum, legality;
    int gx, stashflag;
    int plernum = gamers[gnum].playernum;

    piecenum = gamers[gnum].holding;
    printf("ser: telling %d to drop %d\n", gnum, piecenum);

    compute_piece(piecenum, xp, yp, rot, att);

    legality = legal_placement(gnum, piecenum, &stashflag);

    switch (legality)
    {
	case PLACE_OK:
	    if (optionval[OPT_USE_PLACING_MODE])
		gamers[gnum].placing = 0;
	    break;
	case PLACE_CONFLICT: {
	    msg_notify msg;
	    msg.type = mtype_notify;	
	    msg.what = NOTIFY_CANTDROP;
	    gsendmsg(gnum, &msg);
	    return; /* disallow placement */
	    }
	case PLACE_CRASH: {
	    msg_piececrash msg;
	    if (optionval[OPT_USE_PLACING_MODE])
		gamers[gnum].placing = 0;
	    msg.type = mtype_piececrash;
	    msg.flag = 1;
	    msg.gamernum = gnum;
	    gamers[gnum].crashed = 1;
	    gamers[gnum].placing = 0;
	    /* broadcast fact of crashing */
	    for (gx=0; gx<numgamers; gx++)
		gsendmsg(gx, &msg);
	    return; /* disallow placement */
	    }
	case PLACE_NODECRASH: {
	    msg_notify msg;
	    msg.type = mtype_notify;	
	    msg.what = NOTIFY_NODECRASH;
	    gsendmsg(gnum, &msg);
	    return; /* disallow placement */
	    }
	case PLACE_ZONEEDGE: {
	    msg_notify msg;
	    msg.type = mtype_notify;	
	    msg.what = NOTIFY_ZONEEDGE;
	    gsendmsg(gnum, &msg);
	    return; /* disallow placement */
	    }
	case PLACE_OFFBOARD: {
	    msg_notify msg;
	    msg.type = mtype_notify;	
	    msg.what = NOTIFY_OFFBOARD;
	    gsendmsg(gnum, &msg);
	    return; /* disallow placement */
	    }
	case PLACE_MELTDOWN: {
	    msg_notify msg;
	    msg.type = mtype_notify;	
	    msg.what = NOTIFY_MELTDOWN;
	    gsendmsg(gnum, &msg);
	    return; /* disallow placement */
	    }
	case PLACE_FOREIGNSTASH: 
	    if (gamers[gnum].crashed) {
		/* stashflag already says which pad; just do it */
		break;
	    }
	    else {
		msg_notify msg;
		msg.type = mtype_notify;	
		msg.what = NOTIFY_FOREIGNSTASH;
		gsendmsg(gnum, &msg);
		return; /* disallow placement */
	    }
	case PLACE_STASH:
	    /* stashflag already says which pad; just do it */
	    break; 
	case PLACE_ATTACK_NOTHING: {
	    msg_notify msg;
	    msg.type = mtype_notify;
	    if (optionval[OPT_ALLOW_SQUANDERS]) {
		msg.what = NOTIFY_SQUANDER_NOTHING;
		gsendmsg(gnum, &msg);
	    }
	    else {
		msg.what = NOTIFY_NOSQUANDER_NOTHING;
		gsendmsg(gnum, &msg);
		return; 
	    }
	    }
	    break; 
	case PLACE_ATTACK_ATTACK: {
	    msg_notify msg;
	    msg.type = mtype_notify;
	    if (optionval[OPT_ALLOW_SQUANDERS]) {
		msg.what = NOTIFY_SQUANDER_ATTACK;
		gsendmsg(gnum, &msg);
	    }
	    else {
		msg.what = NOTIFY_NOSQUANDER_ATTACK;
		gsendmsg(gnum, &msg);
		return; 
	    }
	    }
	    break; 
	case PLACE_ATTACK_SELF: {
	    msg_notify msg;
	    msg.type = mtype_notify;
	    if (optionval[OPT_ALLOW_SQUANDERS]) {
		msg.what = NOTIFY_SQUANDER_SELF;
		gsendmsg(gnum, &msg);
	    }
	    else {
		msg.what = NOTIFY_NOSQUANDER_SELF;
		gsendmsg(gnum, &msg);
		return; 
	    }
	    }
	    break; 
	default:
	    printf("Error: bad case %d in drop_piece!\n", legality);
	    return;
    }

    emplace_piece(piecenum, stashflag);

    /* decrease meltdown count if this is your own piece and it's being put on the board. (We needn't check that it's defensive, since meltdowns have already been screened out above.) */
    if (gamers[gnum].meltstatus && stashflag==STATE_GAME && pieces[piecenum].gamernum == gnum)
	gamers[gnum].meltstatus--;

    if (gamers[gnum].crashed) {
	/* if gamer is crashed but has gotten this far, it's a legal move. Uncrash him. */
	msg_piececrash msg;
	msg.type = mtype_piececrash;
	msg.flag = 0;
	msg.gamernum = gnum;
	gamers[gnum].crashed = 0;
	/* broadcast fact of uncrashing */
	for (gx=0; gx<numgamers; gx++)
	    gsendmsg(gx, &msg);
    }

    msgt.type = mtype_piecestatus;
    msgt.gamernum = gnum;
    msgt.piecenum = piecenum;
    msgt.state = pieces[piecenum].state;
    msgt.attack = pieces[piecenum].attack;
    msgt.x = pieces[piecenum].x;
    msgt.y = pieces[piecenum].y;
    msgt.rot = pieces[piecenum].rot;

    /* send to everybody */
    for (gx=0; gx<numgamers; gx++) 
	gsendmsg(gx, &msgt);

    if (is_game_over()) {
	perform_game_over();
    }
}

int legal_lift(gnum, piecenum)
int gnum, piecenum;
{
    if (pieces[piecenum].state >= 0) /* on a stash pad */
    {
	if (piece_pad_overlap(&(pieces[piecenum]), &(gamers[gnum].pad)) == 1)
	    return LIFT_OK; /* on that player's pad */
	else 
	    return LIFT_FOREIGNSTASH; /* someone else's pad */
    }

    if (pieces[piecenum].attack)
    {
	int ix;
	int attpoints;
	int target;
	int gamtarget;

	target = find_attacked_piece(piecenum);
	if (target == (-1))
	    return LIFT_ATT_NOTARGET; /* attack piece attacking nothing */

	if (pieces[target].attack)
	    return LIFT_ATT_TARGETATT; /* attack piece attacking attack piece */

	gamtarget = pieces[target].gamernum;

	if (gamtarget != gnum)
	    return LIFT_ATT_NOTYOURS; /* someone else's defense piece */
	
	if (gamtarget == pieces[piecenum].gamernum)
	    return LIFT_ATT_OWNCOLOR; /* attack piece attacking own color is stuck */

	attpoints = points_attacking_piece(target, piecenum);
	if (attpoints > pieces[target].size)
	    return LIFT_ATT_OVERICE;
	else
	    return LIFT_ATT_NOTOVERICE;
    }
    return LIFT_DEF_BAD;
}

void grab_piece(gnum, piecenum)
int gnum, piecenum;
{
    msg_piecestatus msg;
    msg_notify msg2;
    int gx;
    int legality;

    legality = legal_lift(gnum, piecenum); /* check for legality */

    switch (legality)
    {
	case LIFT_OK:
	    break; /* just do it */
	case LIFT_FOREIGNSTASH:
	    msg2.type = mtype_notify;	
	    msg2.what = NOTIFY_LIFT_FOREIGNSTASH;
	    gsendmsg(gnum, &msg2);
	    return;
	case LIFT_ATT_OVERICE:
	    break;
	case LIFT_DEF_BAD:
	    msg2.type = mtype_notify;	
	    msg2.what = NOTIFY_LIFT_DEF_BAD;
	    gsendmsg(gnum, &msg2);
	    return;
	case LIFT_ATT_NOTARGET:
	    msg2.type = mtype_notify;	
	    msg2.what = NOTIFY_LIFT_ATT_NOTARGET;
	    gsendmsg(gnum, &msg2);
	    return;
	case LIFT_ATT_TARGETATT:
	    msg2.type = mtype_notify;	
	    msg2.what = NOTIFY_LIFT_ATT_TARGETATT;
	    gsendmsg(gnum, &msg2);
	    return;
	case LIFT_ATT_NOTOVERICE:
	    msg2.type = mtype_notify;	
	    msg2.what = NOTIFY_LIFT_ATT_NOTOVERICE;
	    gsendmsg(gnum, &msg2);
	    return;
	case LIFT_ATT_OWNCOLOR:
	    msg2.type = mtype_notify;	
	    msg2.what = NOTIFY_LIFT_ATT_OWNCOLOR;
	    gsendmsg(gnum, &msg2);
	    return;
	case LIFT_ATT_NOTYOURS:
	    msg2.type = mtype_notify;	
	    msg2.what = NOTIFY_LIFT_ATT_NOTYOURS;
	    gsendmsg(gnum, &msg2);
	    return;
	default:
	    printf("Error: bad case %d in grab_piece!\n", legality);
	    return;  
    }

    lift_piece(gnum, piecenum);

    msg.type = mtype_piecestatus;
    msg.gamernum = gnum;
    msg.piecenum = piecenum;
    msg.state = pieces[piecenum].state; 
    msg.attack = pieces[piecenum].attack; 
    msg.x = pieces[piecenum].x;
    msg.y = pieces[piecenum].y;
    msg.rot = pieces[piecenum].rot;

    /* send to all gamers*/
    for (gx=0; gx<numgamers; gx++) 
	gsendmsg(gx, &msg);
}

void handle_lclick(gnum, xp, yp, rot, att)
int gnum;
double xp, yp, rot;
short att;
{
    int piecenum;
    point p;
    int plernum = gamers[gnum].playernum;

    if (gamers[gnum].holding != -1) /* if player is holding a piece */
    {
	if (att == -1)
	    return; /* client's info is out of date */

	drop_piece(gnum, xp, yp, rot, att); /* try to drop it */
	printf( "dropping piece\n");

	return;
    }

    p.x = xp;
    p.y = yp;
    piecenum = find_piece(&p);
    if (piecenum == -1) /* no piece there */
    {
    }
    else
    {
	grab_piece(gnum, piecenum); /* try to grab piece */
    }
}

