#include <stdio.h>
#include <math.h>
#include "ser.h"

static msg_buffer buf;

void enter_plers(), setup_sig();
extern void init_options(), emplace_piece(), compute_zonebounds(), compute_piece();

void init_game()
{
    int ix;

    init_options();
    for (ix=0; ix<MAXPLAYERS; ix++) {
	plers[ix].majormode = MAJMODE_NIL;
    }

    gamemode = GMODE_START;
}

/* set up all board data for starting game */
void init_board()
{
    int gx, ix, sd;
    int snums[4];

    short curpiece;

    gamemode = GMODE_GAME;

    worldxmin = -200.0;
    worldymin = -200.0;
    worldxmax = 200.0;
    worldymax = 200.0;

    board.pts[0].x = -100.0;
    board.pts[0].y = -100.0;
    board.pts[1].x = 100.0;
    board.pts[1].y = -100.0;
    board.pts[2].x = 100.0;
    board.pts[2].y = 100.0;
    board.pts[3].x = -100.0;
    board.pts[3].y = 100.0;
    board.x = 0.0;
    board.y = 0.0;
    compute_zonebounds(&board);

    numgamers = connections;

    gamers = (gamer *)malloc(sizeof(gamer) * numgamers);
    for (gx=0; gx<numgamers; gx++) {
	gamers[gx].playernum = clients[gx]->playernum;
	plers[gamers[gx].playernum].gamernum = gx;
	gamers[gx].sock_num = clients[gx]->sock_num;
	gamers[gx].crashed = 0;
	gamers[gx].placing = 0;
	gamers[gx].icehoused = 0;
	gamers[gx].stashlimit = plers[gamers[gx].playernum].signumpieces / 2 + 1;
	gamers[gx].meltstatus = 2; /* @#@ variants can change this */
	gamers[gx].holding = (-1);
	/*printf("gx %d\n gamers[gx].playernum %d\n plers[gamers[gx].playernum].gamernum %d\n", gx, gamers[gx].playernum, plers[gamers[gx].playernum].gamernum);*/
    }

    numpieces = 0;
    for (gx=0; gx<numgamers; gx++) {
	short pnum = gamers[gx].playernum;
	numpieces += plers[pnum].signumpieces;
    }

    pieces = (piece *)malloc(sizeof(piece) * numpieces); 

    /* moderately keen algorithm for 4-sided board. Puts stash pads along edges of board bounding box. */

    for (ix=0; ix<4; ix++) {
	if (ix < numgamers%4)
	    snums[ix] = numgamers/4 + 1;
	else
	    snums[ix] = numgamers/4;
    }

    curpiece = 0;
    sd = 0;
    ix = 0;
    for (gx=0; gx<numgamers; gx++) {
	double srot, locx, locy, frac;

	frac = (double)(ix+1) / (double)(snums[sd]+1);
	switch (sd) {
	    case 0:
		srot = 0.0;
		locx = board.boundxmin + (board.boundxmax - board.boundxmin) * frac;
		locy = board.boundymax + 20.0;
		break;
	    case 2:
		srot = PI+HALFPI;
		locx = board.boundxmax + 20.0;
		locy = board.boundymax - (board.boundymax - board.boundymin) * frac;
		break;
	    case 1:
		srot = PI;
		locx = board.boundxmax - (board.boundxmax - board.boundxmin) * frac;
		locy = board.boundymin - 20.0;
		break;
	    case 3:
		srot = HALFPI;
		locx = board.boundxmin - 20.0;
		locy = board.boundymin + (board.boundymax - board.boundymin) * frac;
		break;
	}

	setup_sig(gx, &curpiece, locx, locy, srot);
	ix++;
	if (ix==snums[sd]) {
	    ix=0;
	    sd++;
	}
    }

    enter_plers();
}

/* initialize player data at login */
int create_pler()
{
    int ix, jx;

    for (ix=0; plers[ix].majormode!=MAJMODE_NIL; ix++);
    /* ### if ix==limit then return error!!! ### */

    plers[ix].majormode = MAJMODE_NOTIN;
    plers[ix].sig = NULL; 
    for (jx=0; jx<NUMOPTIONS; jx++) {
	plers[ix].reqoption[jx] = 0;
    }
    return ix;
}

/* send player data for starting game */
void enter_plers()
{
    int ix, jx;
    int gx;

    msg_plerin msg;
    msg_pieceplaces *mptr = (msg_pieceplaces *)(&buf);
    msg_zoneplaces *zptr = (msg_zoneplaces *)(&buf);

    for (gx=0; gx<numgamers; gx++) {
	short pnum = gamers[gx].playernum;
	plers[pnum].majormode = MAJMODE_IN;
	plers[pnum].reqoption[OPT_WANTSTART] = 0;
    }

    mptr->type = mtype_pieceplaces;
    mptr->numpieces = numpieces;
    for (ix=0; ix<numpieces; ix++) {
	mptr->p[ix].gamernum = pieces[ix].gamernum; /* this is gamer number, which is the only thing the client understands. */
	mptr->p[ix].size = pieces[ix].size;
	mptr->p[ix].state = pieces[ix].state;
	mptr->p[ix].x = pieces[ix].x;
	mptr->p[ix].y = pieces[ix].y;
	mptr->p[ix].rot = pieces[ix].rot;
	mptr->p[ix].attack = pieces[ix].attack;
    }

    for (gx=0; gx<numgamers; gx++) {
	gesendmsg(gx, mptr, numpieces*sizeof(struct msg_pieceplaces_sub));
    }

    /* one zone for the board, and one for each stash pad */
    zptr->type = mtype_zoneplaces;
    zptr->numzones = numgamers+1;
    zptr->z[0].x = board.x;
    zptr->z[0].y = board.y;
    for (jx=0; jx<4; jx++) {
	zptr->z[0].pts[jx].x = board.pts[jx].x;
	zptr->z[0].pts[jx].y = board.pts[jx].y;
    }
    for (gx=0; gx<numgamers; gx++) {
	zptr->z[gx+1].x = gamers[gx].pad.x;
	zptr->z[gx+1].y = gamers[gx].pad.y;
	for (jx=0; jx<4; jx++) {
	    zptr->z[gx+1].pts[jx].x = gamers[gx].pad.pts[jx].x;
	    zptr->z[gx+1].pts[jx].y = gamers[gx].pad.pts[jx].y;
	}
    }
    for (gx=0; gx<numgamers; gx++) {
	gesendmsg(gx, zptr, zptr->numzones*sizeof(struct msg_zoneplaces_sub));
    }

    for (gx=0; gx<numgamers; gx++) {
	msg.type = mtype_plerin;
	msg.numgamers = numgamers;
	msg.gamernum = gx;
	msg.boardxmin = board.boundxmin;
	msg.boardymin = board.boundymin;
	msg.boardxmax = board.boundxmax;
	msg.boardymax = board.boundymax;
	msg.worldxmin = worldxmin;
	msg.worldymin = worldymin;
	msg.worldxmax = worldxmax;
	msg.worldymax = worldymax;
	gsendmsg(gx, &msg);
    }
}

/* create a player's stash pad and drop all her pieces on it. locx and locy refer to the center of the pad. */
void setup_sig(gamernum, pcurpiece, locx, locy, stashrot)
short gamernum, *pcurpiece;
double locx, locy, stashrot;
{
#define STASHRATIO (ZEROPIECESIZE * 7.0)
    short ix, att;
    point p, pp, stashsize;
    double rot;
    short curpiece = *pcurpiece;
    pler *pt = &(plers[gamers[gamernum].playernum]);
    zone *pad;

    stashsize.x = pt->sigsize.x * STASHRATIO;
    stashsize.y = pt->sigsize.y * STASHRATIO;

    /* subtract an icky offset to get the pad's 0,0 corner. */
    p.x = stashsize.x / -2.0;
    p.y = stashsize.y / -2.0;
    locx += (p.x*cos(stashrot) - p.y*sin(stashrot));
    locy += (p.x*sin(stashrot) + p.y*cos(stashrot));

    for (ix=0; ix<pt->signumpieces; ix++)
    {
	pieces[curpiece].size = pt->sig[ix].size;
	pieces[curpiece].gamernum = gamernum;
	pieces[curpiece].heldby = -1;
	p.x = pt->sig[ix].x * STASHRATIO;
	p.y = pt->sig[ix].y * STASHRATIO;
	pp.x = p.x*cos(stashrot) - p.y*sin(stashrot);
	pp.y = p.x*sin(stashrot) + p.y*cos(stashrot);
	rot = pt->sig[ix].rot + stashrot;
	att = pt->sig[ix].attack;
	compute_piece(curpiece, pp.x + locx, pp.y + locy, rot, att);
	emplace_piece(curpiece, gamernum);	
	curpiece ++;
    }
    *pcurpiece = curpiece;

    /* set up pad field in player structure */
    pad = &(gamers[gamernum].pad);
    pad->pts[0].x = locx + 0.0*cos(stashrot) - 0.0*sin(stashrot);
    pad->pts[0].y = locy + 0.0*sin(stashrot) + 0.0*cos(stashrot);
    pad->pts[1].x = locx + stashsize.x*cos(stashrot) - 0.0*sin(stashrot);
    pad->pts[1].y = locy + stashsize.x*sin(stashrot) + 0.0*cos(stashrot);
    pad->pts[2].x = locx + stashsize.x*cos(stashrot) - stashsize.y*sin(stashrot);
    pad->pts[2].y = locy + stashsize.x*sin(stashrot) + stashsize.y*cos(stashrot);
    pad->pts[3].x = locx + 0.0*cos(stashrot) - stashsize.y*sin(stashrot);
    pad->pts[3].y = locy + 0.0*sin(stashrot) + stashsize.y*cos(stashrot);
    p.x = 0.5 * stashsize.x;
    p.y = 0.5 * stashsize.y;
    pad->x = locx + p.x*cos(stashrot) - p.y*sin(stashrot);
    pad->y = locy + p.x*sin(stashrot) + p.y*cos(stashrot);

    compute_zonebounds(pad);
}

