#include <stdio.h>
#include <math.h>
#include "ser.h"

double intersect_seg_dist();

/* return 0 if piece is outside, 1 if piece is inside, 2 if it overlaps. Assumes compute_piece() has been called for the piece. Fails if piece completely surrounds pad. */
int piece_pad_overlap(g1, g2)
piece *g1;
zone *g2;
{
    int sides1, sides2;
    int ix, jx, iy, jy;
    point pvec, qvec, pc;

    if (g1->boundxmin > g2->boundxmax
	 || g1->boundymin > g2->boundymax
	 || g2->boundxmin > g1->boundxmax
	 || g2->boundymin > g1->boundymax)
	return 0;

    if (!(g1->attack)) sides1 = 4;
    else sides1 = 3;

    sides2 = 4;

    for (ix=0; ix<sides1; ix++) {
	iy = (ix+1) % sides1;
	pvec.x = g1->pts[iy].x - g1->pts[ix].x;
	pvec.y = g1->pts[iy].y - g1->pts[ix].y;
	for (jx=0; jx<sides2; jx++) {
	    jy = (jx+1) % sides2;
	    qvec.x = g2->pts[jy].x - g2->pts[jx].x;
	    qvec.y = g2->pts[jy].y - g2->pts[jx].y;
	    if (intersect_seg(&(g1->pts[ix]), &pvec, &(g2->pts[jx]), &qvec))
		return 2;
	}
    }

    pc.x = g1->x;
    pc.y = g1->y;
    if (inside_stashpad(g2, &pc)) return 1;

    return 0;
}

/* return TRUE if they do. Assumes compute_piece() has been called for both. */
int piece_overlap(g1, g2)
piece *g1, *g2;
{
    int sides1, sides2;
    int ix, jx, iy, jy;
    point pvec, qvec, pc;

    if (g1->boundxmin > g2->boundxmax
	 || g1->boundymin > g2->boundymax
	 || g2->boundxmin > g1->boundxmax
	 || g2->boundymin > g1->boundymax)
	return 0;

    if (!(g1->attack)) sides1 = 4;
    else sides1 = 3;
    if (!(g2->attack)) sides2 = 4;
    else sides2 = 3;

    for (ix=0; ix<sides1; ix++) {
	iy = (ix+1) % sides1;
	pvec.x = g1->pts[iy].x - g1->pts[ix].x;
	pvec.y = g1->pts[iy].y - g1->pts[ix].y;
	for (jx=0; jx<sides2; jx++) {
	    jy = (jx+1) % sides2;
	    qvec.x = g2->pts[jy].x - g2->pts[jx].x;
	    qvec.y = g2->pts[jy].y - g2->pts[jx].y;
	    if (intersect_seg(&(g1->pts[ix]), &pvec, &(g2->pts[jx]), &qvec))
		return 1;
	}
    }

    pc.x = g2->x;
    pc.y = g2->y;
    if (inside_polygon(g1, &pc)) return 1;

    pc.x = g1->x;
    pc.y = g1->y;
    if (inside_polygon(g2, &pc)) return 1;

    return 0;
}

/* return TRUE if point p1 is inside piece gon */
int inside_polygon(gon, p1)
piece *gon;
point *p1;
{
    point pvec, qvec;
    int numsides;
    int ix;

    pvec.x = gon->x - p1->x;
    pvec.y = gon->y - p1->y;

    if (gon->attack)
	numsides = 3;
    else
	numsides = 4;

    for (ix=0; ix<numsides; ix++) {
	qvec.x = gon->pts[(ix+1) % numsides].x - gon->pts[ix].x;
	qvec.y = gon->pts[(ix+1) % numsides].y - gon->pts[ix].y;

	if (intersect_seg(p1, &pvec, &(gon->pts[ix]), &qvec))
	    return 0;
    }

    return 1;
}

/* return closest contact of vector p to piece gon. No boundbox checking. */
double dist_to_polygon(gon, p1, pvec)
piece *gon;
point *p1, *pvec;
{
    point qvec;
    int numsides;
    int ix;
    double sofar = -1.0, dist;

    if (gon->attack)
	numsides = 3;
    else
	numsides = 4;

    for (ix=0; ix<numsides; ix++) {
	qvec.x = gon->pts[(ix+1) % numsides].x - gon->pts[ix].x;
	qvec.y = gon->pts[(ix+1) % numsides].y - gon->pts[ix].y;

	dist = intersect_seg_dist(p1, pvec, &(gon->pts[ix]), &qvec);
	if (dist>=0.0 && (sofar==(-1.0) || dist<sofar)) {
	    sofar = dist;
	}
    }

    return sofar;
}

/* return TRUE if point p1 is inside pad */
int inside_stashpad(pad, p1)
zone *pad;
point *p1;
{
    point pvec, qvec;
    int numsides;
    int ix;

    pvec.x = pad->x - p1->x;
    pvec.y = pad->y - p1->y;

    numsides = 4;

    for (ix=0; ix<numsides; ix++) {
	qvec.x = pad->pts[(ix+1) % numsides].x - pad->pts[ix].x;
	qvec.y = pad->pts[(ix+1) % numsides].y - pad->pts[ix].y;

	if (intersect_seg(p1, &pvec, &(pad->pts[ix]), &qvec))
	    return 0;
    }

    return 1;
}

/* return TRUE if two segments intersect.  */
int intersect_seg(p1, pvec, q1, qvec)
point *p1, *pvec, *q1, *qvec;
{
    point nq1, nqvec, np1;
    double pmin, pmax, qmin, qmax;
    double skewf;

    /*printf("%.1f %.1f  %.1f %.1f : %.1f %.1f  %.1f %.1f\n", p1->x, p1->y, pvec->x, pvec->y, q1->x, q1->y, qvec->x, qvec->y);*/

    if (pvec->x==0.0) {
	/* p vector is vertical */
	if (qvec->x==0.0) {
	    /* both vertical */

	    if (q1->x != p1->x)
		return 0;

	    if (pvec->y >= 0.0) {
		pmin = p1->y;
		pmax = p1->y + pvec->y;
	    }
	    else {
		pmin = p1->y + pvec->y;
		pmax = p1->y;
	    }
	    if (qvec->y >= 0.0) {
		qmin = q1->y;
		qmax = q1->y + qvec->y;
	    }
	    else {
		qmin = q1->y + qvec->y;
		qmax = q1->y;
	    }
	    if ((pmin>=qmin && pmin<=qmax) 
		|| (qmin>=pmin && qmin<=pmax))
		return 1;
	    else
		return 0;
	}

	skewf = qvec->y / qvec->x;
	np1.x = p1->x;
	np1.y = p1->y - skewf*(p1->x - q1->x);
	/* npvec = pvec, since it's vertical */

	if (qvec->x >= 0.0) {
	    qmin = q1->x;
	    qmax = q1->x + qvec->x;
	}
	else {
	    qmin = q1->x + qvec->x;
	    qmax = q1->x;
	}
	if (pvec->y >= 0.0) {
	    pmin = np1.y;
	    pmax = np1.y + pvec->y;
	}
	else {
	    pmin = np1.y + pvec->y;
	    pmax = np1.y;
	}
	if ((np1.x>=qmin && np1.x<=qmax) 
	    && (q1->y>=pmin && q1->y<=pmax))
	    return 1;
	else
	    return 0;
    }

    skewf = pvec->y / pvec->x;
    nq1.x = q1->x;
    nq1.y = q1->y - skewf*(q1->x - p1->x);
    nqvec.x = qvec->x;
    nqvec.y = qvec->y - skewf*(qvec->x);
    /* pvec->y = 0.0 */

    if (nqvec.y==0.0) {
	/* vectors parallel */

	if (nq1.y != p1->y)
	    return 0;

	if (pvec->x >= 0.0) {
	    pmin = p1->x;
	    pmax = p1->x + pvec->x;
	}
	else {
	    pmin = p1->x + pvec->x;
	    pmax = p1->x;
	}
	if (nqvec.x >= 0.0) {
	    qmin = nq1.x;
	    qmax = nq1.x + nqvec.x;
	}
	else {
	    qmin = nq1.x + nqvec.x;
	    qmax = nq1.x;
	}
	if ((pmin>=qmin && pmin<=qmax) 
	    || (qmin>=pmin && qmin<=pmax))
	    return 1;
	else
	    return 0;
    }

    /* vectors not parallel */
    skewf = nqvec.x / nqvec.y;
    np1.x = p1->x - skewf*(p1->y - nq1.y);
    np1.y = p1->y;
    /* nqvec.x = 0.0 */

    if (pvec->x >= 0.0) {
	pmin = np1.x;
	pmax = np1.x+pvec->x;
    }
    else {
	pmin = np1.x+pvec->x;
	pmax = np1.x;
    }

    if (nqvec.y >= 0.0) {
	qmin = nq1.y;
	qmax = nq1.y+nqvec.y;
    }
    else {
	qmin = nq1.y+nqvec.y;
	qmax = nq1.y;
    }

    if ((nq1.x>=pmin && nq1.x<=pmax) 
	 && (np1.y>=qmin && np1.y<=qmax))
	return 1;
    else
	return 0;
}

/* return dist if two segments intersect, -1.0 if not */
double intersect_seg_dist(p1, pvec, q1, qvec)
point *p1, *pvec, *q1, *qvec;
{
    point nq1, nqvec, np1, npvec;
    double pmin, pmax, qmin, qmax;
    double skewf;

    /*printf("%.1f %.1f  %.1f %.1f : %.1f %.1f  %.1f %.1f\n", p1->x, p1->y, pvec->x, pvec->y, q1->x, q1->y, qvec->x, qvec->y);*/

    if (pvec->x==0.0) {
	/* p vector is vertical */
	if (qvec->x==0.0) {
	    /* both vertical */

	    if (q1->x != p1->x)
		return -1.0;

	    if (qvec->y >= 0.0) {
		qmin = q1->y;
		qmax = q1->y + qvec->y;
	    }
	    else {
		qmin = q1->y + qvec->y;
		qmax = q1->y;
	    }

	    if (pvec->y >= 0.0) {
		npvec.y = pvec->y;
	    }
	    else {
		npvec.y = -pvec->y;
		/* switch and mirror around p1->y */
		pmax = p1->y - (qmin - p1->y); 
		qmin = p1->y - (qmax - p1->y);
		qmax = pmax;
	    }

	    if (qmin <= p1->y && qmax >= p1->y)
		return 0.0;

	    if (qmax < p1->y || qmin > p1->y+npvec.y) /* minus because of mirroring */
		return -1.0;

	    return qmin - p1->y; /* true distance */
	}

	/* only p vertical */
	skewf = qvec->y / qvec->x;
	np1.x = p1->x;
	np1.y = p1->y - skewf*(p1->x - q1->x);
	/* npvec = pvec, since it's vertical */

	if (qvec->x >= 0.0) {
	    qmin = q1->x;
	    qmax = q1->x + qvec->x;
	}
	else {
	    qmin = q1->x + qvec->x;
	    qmax = q1->x;
	}
	if (pvec->y >= 0.0) {
	    if ((np1.x>=qmin && np1.x<=qmax) 
		&& (q1->y>=np1.y && q1->y<=(np1.y+pvec->y)))
		return q1->y - np1.y; /* true distance */
	    else
		return -1.0;
	}
	else {
	    if ((np1.x>=qmin && np1.x<=qmax) 
		&& (q1->y>=(np1.y+pvec->y) && q1->y<=np1.y))
		return np1.y - q1->y; /* true distance */
	    else
		return -1.0;
	}
    }

    skewf = pvec->y / pvec->x;
    nq1.x = q1->x;
    nq1.y = q1->y - skewf*(q1->x - p1->x);
    nqvec.x = qvec->x;
    nqvec.y = qvec->y - skewf*(qvec->x);
    /* pvec->y = 0.0 */

    if (nqvec.y==0.0) {
	/* vectors parallel */

	if (nq1.y != p1->y)
	    return -1.0;

	if (nqvec.x >= 0.0) {
	    qmin = nq1.x;
	    qmax = nq1.x + nqvec.x;
	}
	else {
	    qmin = nq1.x + nqvec.x;
	    qmax = nq1.x;
	}

	if (pvec->x >= 0.0) {
	    npvec.x = pvec->x;
	}
	else {
	    npvec.x = -pvec->x;
	    /* switch and mirror around p1->x */
	    pmax = p1->x - (qmin - p1->x); 
	    qmin = p1->x - (qmax - p1->x);
	    qmax = pmax;
	}

	if (qmin <= p1->x && qmax >= p1->x)
	    return 0.0;

	if (qmax < p1->x || qmin > p1->x+npvec.x) /* minus because of mirroring */
	    return -1.0;

	return qmin - p1->x; /* distance / sin(theta), where theta is angle of P to horizontal */
    }

    /* vectors not parallel */
    skewf = nqvec.x / nqvec.y;
    np1.x = p1->x - skewf*(p1->y - nq1.y);
    np1.y = p1->y;
    /* nqvec.x = 0.0; pvec.x unchanged, pvec.y still 0 */

    if (nqvec.y >= 0.0) {
	qmin = nq1.y;
	qmax = nq1.y+nqvec.y;
    }
    else {
	qmin = nq1.y+nqvec.y;
	qmax = nq1.y;
    }

    if (pvec->x >= 0.0) {
	pmin = np1.x;
	pmax = np1.x+pvec->x;
	if ((nq1.x>=pmin && nq1.x<=pmax) 
	    && (np1.y>=qmin && np1.y<=qmax))
	    return nq1.x - pmin; /* distance / sin(theta), where theta is angle of P to horizontal */
	else
	    return -1.0;
    }
    else {
	pmin = np1.x+pvec->x;
	pmax = np1.x;
	if ((nq1.x>=pmin && nq1.x<=pmax) 
	    && (np1.y>=qmin && np1.y<=qmax))
	    return pmax - nq1.x; /* distance / sin(theta), where theta is angle of P to horizontal */
	else
	    return -1.0;
    }
}

/* use arguments to compute x, y, rot, attack, pts[] and bounding box of piece. Does *not* affect state */
void compute_piece(pnum, xp, yp, rot, attack)
int pnum;
double xp, yp, rot;
short attack;  
{
    int npoints, ix;
    piece *pt = &(pieces[pnum]);
    double rad = ((double)(pieces[pnum].size)*PIECESIZEDIFF + ZEROPIECESIZE);
    
    pt->x = xp;
    pt->y = yp;
    pt->rot = rot;
    pt->attack = attack;

    if (!attack) {
	double xoff = rad*cos(rot);
	double yoff = rad*sin(rot);

	pt->pts[0].x = (xp+xoff);
	pt->pts[0].y = (yp+yoff);
	pt->pts[1].x = (xp-yoff);
	pt->pts[1].y = (yp+xoff);
	pt->pts[2].x = (xp-xoff);
	pt->pts[2].y = (yp-yoff);
	pt->pts[3].x = (xp+yoff);
	pt->pts[3].y = (yp-xoff);

	npoints = 4;

    }
    else { /* attack piece */
	double radtip = rad * 1.673234098593571;
	double radback = rad * 1.095412290954442;
	double radatt = rad * 4.183085246483928; /* 2.5*radtip */
	
	double xtoff = radtip*cos(rot);
	double ytoff = radtip*sin(rot);
	double xboff1 = radback*cos(rot+2.439893204463398);
	double yboff1 = radback*sin(rot+2.439893204463398);
	double xboff2 = radback*cos(rot+3.843292102716188);
	double yboff2 = radback*sin(rot+3.843292102716188);
	double xaoff = radatt*cos(rot);
	double yaoff = radatt*sin(rot);
	pt->pts[0].x = (xp+xtoff);
	pt->pts[0].y = (yp+ytoff);
	pt->pts[1].x = (xp+xboff1);
	pt->pts[1].y = (yp+yboff1);
	pt->pts[2].x = (xp+xboff2);
	pt->pts[2].y = (yp+yboff2);
	pt->pts[3].x = (xp+xaoff); /* tip of attack line */
	pt->pts[3].y = (yp+yaoff);

	npoints = 3; /* not counting tip of attack line */
    }

    /* find bounding box */
    pt->boundxmin = pt->boundxmax = pt->pts[0].x;
    pt->boundymin = pt->boundymax = pt->pts[0].y;
    for (ix = 1; ix < npoints; ix++)
    {
	if (pt->pts[ix].x > pt->boundxmax)
	    pt->boundxmax = pt->pts[ix].x;
	if (pt->pts[ix].y > pt->boundymax)
	    pt->boundymax = pt->pts[ix].y;
	if (pt->pts[ix].x < pt->boundxmin)
	    pt->boundxmin = pt->pts[ix].x;
	if (pt->pts[ix].y < pt->boundymin)
	    pt->boundymin = pt->pts[ix].y;	
    }
}

void compute_zonebounds(zn)
zone *zn;
{
    int ix;

    zn->boundxmin = zn->boundxmax = zn->pts[0].x;
    zn->boundymin = zn->boundymax = zn->pts[0].y;
    for (ix = 1; ix < 4; ix++)
    {
	if (zn->pts[ix].x > zn->boundxmax)
	    zn->boundxmax = zn->pts[ix].x;
	if (zn->pts[ix].y > zn->boundymax)
	    zn->boundymax = zn->pts[ix].y;
	if (zn->pts[ix].x < zn->boundxmin)
	    zn->boundxmin = zn->pts[ix].x;
	if (zn->pts[ix].y < zn->boundymin)
	    zn->boundymin = zn->pts[ix].y;	
    }
}

int point_in_piece(piecenum, pt)
int piecenum;
point *pt;
{
    /* exclude pieces held by players (to avoid overlapping-piece conflicts) */
    if (pieces[piecenum].state == STATE_HELD)
	return 0; 

    /* check bounding box */
    if ( pt->x > pieces[piecenum].boundxmax ||
	 pt->x < pieces[piecenum].boundxmin ||
	 pt->y > pieces[piecenum].boundymax ||
	 pt->y < pieces[piecenum].boundymin)
	return 0;

    /* now check more carefully */
    if (!inside_polygon(&(pieces[piecenum]), pt)) {
	return 0;
    }

    return 1;
}

int find_piece(pt) /* which piece (if any) includes those coordinates? */
point *pt;
{
    int piecenum;
    int ix;
    
    piecenum = -1; /* assume no piece */
    for (ix = 0; ix<numpieces; ix++)
    {
	if (point_in_piece(ix, pt))
	    piecenum = ix;
    }

    /*printf("found piece %d\n", piecenum);*/
    return piecenum;
}

/* find which piece (-1 if none) is being attacked by the given attack piece */
int find_attacked_piece(ap)
int ap;
{
    int ix;
    double boundxmin, boundxmax, boundymin, boundymax;
    int piecenum = -1;
    double dist, sofar=0.0;
    point *pt = pieces[ap].pts;
    point pvec;

    if (pt[0].x < pt[3].x) {
	boundxmin = pt[0].x;
	boundxmax = pt[3].x;
    }
    else  {
	boundxmin = pt[3].x;
	boundxmax = pt[0].x;
    }

    if (pt[0].y < pt[3].y) {
	boundymin = pt[0].y;
	boundymax = pt[3].y;
    }
    else  {
	boundymin = pt[3].y;
	boundymax = pt[0].y;
    }

    pvec.x = pt[3].x - pt[0].x;
    pvec.y = pt[3].y - pt[0].y;

    for (ix=0; ix<numpieces; ix++) 
	if (ix != ap) {
	    piece *gp = &pieces[ix];
	    if (!(gp->boundxmin > boundxmax
		  || gp->boundymin > boundymax
		  || boundxmin > gp->boundxmax
		  || boundymin > gp->boundymax)) {
		dist = dist_to_polygon(gp, &pt[0], &pvec);
		if (dist>=0.0 && (piecenum==(-1) || dist<sofar)) {
		    piecenum = ix;
		    sofar = dist;
		}
	    }
	}

    /* if piecenum is not on board (held or stashpad), no good */
    if (piecenum!=(-1) && pieces[piecenum].state!=STATE_GAME)
	return (-1);

    return piecenum;
}

/* is targ attacked by attack piece ap? */
int is_attacked_piece(ap, targ)
int ap, targ;
{
    int ix;
    double boundxmin, boundxmax, boundymin, boundymax;
    int piecenum = -1;
    double dist, sofar=0.0;
    point *pt = pieces[ap].pts;
    point pvec;
    piece *gp;

    if (targ==ap) return 0;

    if (pt[0].x < pt[3].x) {
	boundxmin = pt[0].x;
	boundxmax = pt[3].x;
    }
    else  {
	boundxmin = pt[3].x;
	boundxmax = pt[0].x;
    }

    if (pt[0].y < pt[3].y) {
	boundymin = pt[0].y;
	boundymax = pt[3].y;
    }
    else  {
	boundymin = pt[3].y;
	boundymax = pt[0].y;
    }

    pvec.x = pt[3].x - pt[0].x;
    pvec.y = pt[3].y - pt[0].y;

    gp = &pieces[targ];
    if (!(gp->boundxmin > boundxmax
	   || gp->boundymin > boundymax
	   || boundxmin > gp->boundxmax
	   || boundymin > gp->boundymax)) {
	dist = dist_to_polygon(gp, &pt[0], &pvec);
	if (dist>=0.0 && (piecenum==(-1) || dist<sofar)) {
	    piecenum = targ;
	    sofar = dist;
	}
    }
    if (piecenum != targ) return 0;

    for (ix=0; ix<numpieces; ix++) 
	if (ix != ap && ix != targ) {
	    gp = &pieces[ix];
	    if (!(gp->boundxmin > boundxmax
		  || gp->boundymin > boundymax
		  || boundxmin > gp->boundxmax
		  || boundymin > gp->boundymax)) {
		dist = dist_to_polygon(gp, &pt[0], &pvec);
		if (dist>=0.0 && (piecenum==(-1) || dist<sofar)) {
		    piecenum = ix;
		    sofar = dist;
		}
	    }
	}

    if (piecenum != targ) return 0;

    return 1;
}

/* how many points are validly attacking defensive piece fnum? except marks a piece to ignore, or -1 to ignore ignorance */
int points_attacking_piece(fnum, except)
int fnum, except;
{
    int attpoints = 0; /* discount current piece */
    int ix;

    for (ix=0; ix<numpieces; ix++) 
	if (ix!=except 
	    && pieces[ix].state == STATE_GAME
	    && pieces[ix].attack 
	    && pieces[ix].gamernum != pieces[fnum].gamernum
	    && is_attacked_piece(ix, fnum)) {
	    attpoints += pieces[ix].size;
	}

    return attpoints;
}

