#include <stdio.h>
#include "ser.h"
#ifdef SOLARIS
#define memcpy memmove
#endif

extern void init_board(), take_cupdate(), enter_plers(), handle_lclick();

void process_buf(ptr, msglen)
char *ptr;
int msglen;
{
    int gx;
    msg_generic *genptr;
    int pnum = clients[active_client]->playernum;
    int gnum = plers[pnum].gamernum; /* don't use this outside a game */
    genptr = (msg_generic *)ptr;

    switch (genptr->type) {
	case mtype_generic:
	    {
	    printf("got GENERIC\n");
	    }
	    break;
	case mtype_crash: {
	    printf("server going down!\n");
	    exit(-1);
	    }
	    break;
	case mtype_ping: {
	    msg_reping msg;
	    msg.type = mtype_reping;
	    {
		int ix;
		for (ix=0; ix<numpieces; ix++) {
		    printf("%.2f, %.2f  %.2f  g=%d, a=%d s=%d, h=%d\n", pieces[ix].x, pieces[ix].y, pieces[ix].gamernum, pieces[ix].rot, pieces[ix].attack, pieces[ix].state, pieces[ix].heldby);
		}
	    }
	    sendmsg(active_client, &msg);
	    }
	    break;
	case mtype_reping: {
	    printf("got reping!\n");
	    }
	    break;

	case mtype_changeoption: {
	    msg_changeoption *msg = (msg_changeoption *)ptr;
	    change_option(pnum, msg->opnum, msg->newval);
	    }
	    break;

	case mtype_cupdate: 
	    if (gamemode==GMODE_GAME) {
		msg_cupdate *msg = (msg_cupdate *)ptr;
		take_cupdate(msg);
	    }
	    break;

	case mtype_lclick: 
	    if (gamemode==GMODE_GAME) {
		msg_lclick *msg = (msg_lclick *)ptr;
		handle_lclick(gnum, (double)msg->x, (double)msg->y, (double)msg->rot, (short)msg->attack);
	    }
	    break;

	case mtype_sbar: 
	    if (gamemode==GMODE_GAME) {
		msg_sbar *msg = (msg_sbar *)ptr;
		int piecenum = msg->piecenum;

		if( gamers[gnum].crashed ) {
		    
		    msg_notify disallow;

		    /*Tried to enter placing mode while crashed -- refuse */
		    disallow.type = mtype_notify;
		    disallow.what = NOTIFY_CANTPLACECRASHED;
		    gsendmsg(gnum, &disallow);
		    gamers[gnum].placing = 0;
		    return; /* disallow entry into placing mode */
		}

		gamers[gnum].placing = !gamers[gnum].placing;
		if (gamers[gnum].placing)
		{
		    int ix;
		    compute_piece(piecenum, pieces[piecenum].x, pieces[piecenum].y, pieces[piecenum].rot, pieces[piecenum].attack );
		    for (ix = 0; ix<numpieces; ix++)
		    {
			if (ix != piecenum && ( (pieces[ix].state == STATE_GAME) || ((pieces[ix].state == STATE_HELD) && (gamers[pieces[ix].heldby].placing)) ) )
			{
			    if (piece_overlap(&(pieces[piecenum]), &(pieces[ix])))
			    {
				msg_piececrash msg;

				if (pieces[ix].state == STATE_HELD)
				{
				    msg_notify disallow;

				    /*Tried to enter placing mode on top other piece in placing mode--refuse. */
				    disallow.type = mtype_notify;
				    disallow.what = NOTIFY_CANTPLACE;
				    gsendmsg(gnum, &disallow);
				    gamers[gnum].placing = 0;
				    return; /* disallow entry into placing mode */
				}
				/* Crashed into played piece while placing */
				gamers[gnum].placing = 0;
				msg.type = mtype_piececrash;
				msg.flag = 1;
				msg.gamernum = gnum;
				gamers[gnum].crashed = 1;
				gamers[gnum].placing = 0;
				/* broadcast fact of crashing */
				for (gx=0; gx<numgamers; gx++)
				    gsendmsg(gx, &msg);
				return; /* disallow placement */
			    }
			}
		    }
		}
	    }
	    break;

	case mtype_reqicehouse:
	    handle_icehouse(gnum);
	    break;
	case mtype_askresign: 
	    if (gamemode==GMODE_GAME) {
/*
		msg_resign msg;
		msg.type = mtype_resign;
		sendmsg(active_client, &msg);
		plers[pnum].majormode = MAJMODE_NOTIN;
		gx = plers[pnum].gamernum;
		gamers[gx].playernum = (-1);
*/
		perform_game_over();
	    }
	    break;

	case mtype_signature: {
	    msg_signature *msg = (msg_signature *)ptr;

	    if (plers[pnum].sig) free(plers[pnum].sig);
	    plers[pnum].sig = (siglist *)malloc(sizeof(siglist) * msg->numpieces);
	    plers[pnum].sigsize.x = msg->x;
	    plers[pnum].sigsize.y = msg->y;
	    plers[pnum].signumpieces = msg->numpieces;
	    memcpy(plers[pnum].sig, msg->sig, (int)(sizeof(siglist) * msg->numpieces));
	    }
	    break;

	default:
	    printf("other message type %d\n", genptr->type);
	    break;
    };
}

