#include <stdio.h>
#ifdef SOLARIS
#include <string.h>
#else
#include <strings.h>
#endif
#include <signal.h>
#include <sys/ioctl.h>
#include <sys/wait.h>
#include <fcntl.h>
#include <sys/errno.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/uio.h>
#include <netinet/in.h>
#include <sys/time.h>
#include <netdb.h>
#include "ser.h"

#ifndef FD_SET		/* for 4.2BSD */
#define FD_SETSIZE      (sizeof(fd_set) * NBBY)
#define	FD_SET(n, p)	((p)->fds_bits[(n)/NFDBITS] |= (1 << ((n) % NFDBITS)))
#define	FD_CLR(n, p)	((p)->fds_bits[(n)/NFDBITS] &= ~(1 << ((n) % NFDBITS)))
#define	FD_ISSET(n, p)	((p)->fds_bits[(n)/NFDBITS] & (1 << ((n) % NFDBITS)))
#define FD_ZERO(p)	memset((char *)(p), 0, sizeof(*(p)))
#endif

#define PORT 3738
#define SERVERTICK (350000)

extern clref create_new_client();
extern clref wait_for();
extern void server_loop(), take_client(), take_incoming(), do_quit();
extern void process_buf();
extern void perror();
extern void free();

int incoming_sock;
clref connections;   /* number of active connections, also first free spot in clients[] */
clref active_client;  /* index into clients[] */
game_client *clients[MAXPLAYERS];
static struct timeval timer;

void init_server()
{
    int ix;
    int portnum = PORT;

    gettimeofday(&timer, (struct timezone *)NULL);

    connections=0;
    for (ix=0; ix<MAXPLAYERS; ix++) clients[ix] = (game_client *)NULL;
    incoming_sock = (-1);
    while (incoming_sock<0 && portnum<PORT+10) {
	incoming_sock = serversock(portnum);
	if (incoming_sock < 0) {
	    fprintf(stderr, "unable to grab port %d\n", portnum);
	    portnum++;
	}  
    }
    if (incoming_sock < 0) {
	fprintf(stderr, "giving up.\n");
	exit(-1);
    }
    (void)printf("I have port %d\n", portnum);
}

void take_incoming(insock) /* the incoming socket is readable; take the connection */
int insock;
{
    int new_sock;
    struct sockaddr found_sa;
    int found_sa_len;
    clref new_client;

    found_sa_len = sizeof(found_sa);
    new_sock = accept(insock, &found_sa, &found_sa_len);
    if ((new_sock) == (-1)) 
	perror("accept");
    else {
	if (gamemode != GMODE_START) {
	    msg_gamerunning msg;
	    msg.type = mtype_gamerunning;
	    msg.length = sizeof(msg_gamerunning);
	    write(new_sock, (char *)(&msg), msg.length); 
	    (void)close(new_sock);
	}

	new_client = create_new_client(new_sock);

	if (new_client == ERROR) {
	    msg_gamefull msg;
	    msg.type = mtype_gamefull;
	    msg.length = sizeof(msg_gamefull);
	    write(new_sock, (char *)(&msg), msg.length); 
	    (void)close(new_sock);
	}
	else {
	    msg_hello msg;
	    int ix;

	    /* send hello */
	    msg.type = mtype_hello;
	    clients[new_client]->playernum = create_pler(); /* ### create player structure with stuff. Set to NOTIN */
	    msg.playernum = clients[new_client]->playernum;
	    for (ix=0; ix<NUMOPTIONS; ix++) {
		msg.optvals[ix] = optionval[ix];
	    }
	    sendmsg(new_client, &msg);

	    /* inform new player of each old player, and each old players of new player */
	    review_all_options();
	    printf("##hello sent\n");
	}
    }
}

clref create_new_client(sock)
int sock;
{
    game_client *ptr;

    if (connections==MAXPLAYERS) return ERROR;
    ptr = (game_client *)malloc(sizeof(game_client));
    if (ptr==(game_client *)NULL) return ERROR;
    ptr->sock_num = sock;
    ptr->host_name[0] = '\0';
    ptr->in_buf[0] = '\0';
    ptr->in_pos = 0;
    ptr->quit_flag = 0;
    clients[connections] = ptr;
    connections++;
    return connections-1;
}

void do_quit(indx) 
clref indx;
{
    clref jx;
    int ix, gx;

    if (indx<0 || indx>=connections) return;

    /* ### remove from all game structures */
    ix = clients[indx]->playernum;
    if (gamemode == GMODE_GAME) {
	gx = plers[ix].gamernum;
	gamers[gx].playernum = (-1);
    }
    plers[ix].majormode = MAJMODE_NIL;

    (void)close(clients[indx]->sock_num);
    free(clients[indx]);
    for(jx=indx; jx<connections-1; jx++) {
	clients[jx] = clients[jx+1];
    };
    clients[jx] = (game_client *)NULL;
    connections--;

    if (connections==0) {
	fprintf(stderr, "xicehouse server: no more players; shutting down.\n");
	exit(0);
    }

    /* ### inform players that this guy is gone */
    review_all_options();
    printf("##goodbye sent\n");
}

clref wait_for(cls, numcon, insock) /* returns a client, or -1 for incoming, or -3 for timeout, or -2 for error */
game_client **cls;
clref numcon;
int insock;
{
    fd_set readbits, writebits, exbits;
    int ret;
    clref fd;
    struct timeval tv;

    while (1) {
	gettimeofday(&tv, (struct timezone *)NULL);
	if ((tv.tv_sec-timer.tv_sec)*1000000 + (tv.tv_usec-timer.tv_usec) > SERVERTICK) {
	    timer.tv_sec = tv.tv_sec;
	    timer.tv_usec = tv.tv_usec;
	    return (-3);
	}
	tv.tv_sec = 0;
	tv.tv_usec = 100000;
	FD_ZERO(&readbits);
	FD_ZERO(&writebits);
	FD_ZERO(&exbits);
	FD_SET(insock, &readbits);
	for (fd=0; fd<numcon; fd++)
	    FD_SET(cls[fd]->sock_num, &readbits);
	ret = select(64, &readbits, &writebits, &exbits, &tv);
	if (ret) {
	    if (FD_ISSET(insock, &readbits)) return (-1);
	    for (fd=0; fd<numcon; fd++)
		if (FD_ISSET(cls[fd]->sock_num, &readbits))
		    return fd;
	    return (-2);
	};
	/* else select timed out ... */
    }
}

void take_client(rcli) /* take data from client */
clref rcli;
{
    int rval, ix, done;
    int msglen;
    game_client *cli = clients[rcli];
    char *ptr;

    rval = read(cli->sock_num, (cli->in_buf+cli->in_pos), (int)(MAXBUF-(cli->in_pos)));
    if (rval < 0) {
	perror("reading stream message");
	cli->in_pos = 0;
    }
    else {
	if (rval == 0) 
	    cli->quit_flag=1;
	else {
	    cli->in_pos += rval;
	    ptr = cli->in_buf;
	    msglen = *((int *)ptr);
	    done = 0;
	    while (!done && ptr+msglen <= cli->in_buf+cli->in_pos) {
		active_client = rcli;
		process_buf(ptr, msglen);
		ptr += msglen;
		if (ptr == cli->in_buf+cli->in_pos) {
		    done = 1;
		}
		else {
		    msglen = *((int *)ptr);
		}
	    }
	    for (ix=0; ix<((cli->in_buf+cli->in_pos)-ptr); ix++)
		cli->in_buf[ix] = ptr[ix];
	    cli->in_pos = ix;
	}
    }
}

/*
** serversock()
**
** Creates an internet socket, binds it to an address, and prepares it for
** subsequent accept() calls by calling listen().
**
** Input: port number desired, or 0 for a random one
** Output: file descriptor of socket, or a negative error
*/
int serversock(port)
int port;
{
    int	sock, x, result;
    struct sockaddr_in server;

    sock = socket(AF_INET, SOCK_STREAM, 0);
    if (sock < 0) {
	perror("attempt to create socket");
	return -errno;
    };

    memset((char *)(&server), 0, sizeof(server));
    server.sin_family = AF_INET;
    server.sin_addr.s_addr = INADDR_ANY;
    server.sin_port = htons(port);

    x = bind(sock, &server, sizeof(server));
    if (x < 0) {
	perror("attempt to bind socket");
	(void)close(sock);
	return -errno;
    }

    result = listen(sock, 5);
    if (result) {
	perror("attempt to listen");
	return (-1);
    };

    return sock;
}

/*
** portnum()
**
** Returns the internet port number for a socket.
**
** Input: file descriptor of socket
** Output: inet port number
*/
int portnum(fd)
int fd;
{
    int	length, err;
    struct sockaddr_in address;

    length = sizeof(address);
    err = getsockname(fd, &address, &length);
    if (err < 0)
	return -errno;

    return ntohs(address.sin_port);
}

char *get_sock_host(i)
int i;
{
    struct sockaddr_in from;
    int fromlen = sizeof(from);
    struct hostent *host;
    struct in_addr addr;
    
    if (getpeername(i, &from, &fromlen) < 0) {
	(void)printf("###error getting host address on pipe %d\n",i);
	return NULL;
    }
	
    addr = from.sin_addr;
    if ((host =	gethostbyaddr(&addr, sizeof(addr), AF_INET)) == NULL) {
	(void)printf("### error getting host name\n");
	return NULL;
    }
    
    return host->h_name;
}

real_bugsendmsg(rcli, obj, length) 
clref rcli;
char *obj;
int length;
{ 
    msg_generic *ptr = (msg_generic *)obj;
    ptr->length = length; 
    write(clients[rcli]->sock_num, obj, length); 
}
