#include <stdio.h>
#include <math.h>
#include "ser.h"

static msg_buffer buf;

int is_game_over()
{   
    int ix;

    for (ix=0; ix<numpieces; ix++) {
	if (pieces[ix].state != STATE_GAME)
	    return 0;
    }

    return 1;
}

/* take everything down.  */
void final_board()
{
    msg_gameover *mptr = (msg_gameover *)&buf;
    int ix, cx, gx;
    int bestscore, winner;

    bestscore = 0;
    winner = (-1);
    for (ix=0; ix<numgamers; ix++) {
	if (gamers[ix].score > bestscore) {
	    bestscore = gamers[ix].score;
	    winner = ix;
	}
	else if (gamers[ix].score == bestscore) {
	    winner = (-1);
	}
    }
    if (winner == (-1)) {
	printf("## tied game\n");
    }
    else {
	printf("## gamer %d won! %d points.\n", winner, bestscore);
    }

    mptr->type = mtype_gameover;
    for (ix=0; ix<numgamers; ix++)
	mptr->scores[ix] = gamers[ix].score;
    for (ix=0; ix<numpieces; ix++)
	mptr->piecescores[ix] = pieces[ix].score;
    mptr->winner = winner;

    for (gx=0; gx<numgamers; gx++) {
	gesendmsg(gx, mptr, numpieces*sizeof(short));
    }

    gamemode = GMODE_START;
    optionval[OPT_WANTSTART] = 0;
    for (cx=0; cx<connections; cx++) {
	ix = clients[cx]->playernum;
	plers[ix].reqoption[OPT_WANTSTART] = 0;
    }

    free(pieces);
    pieces = NULL;
    free(gamers);
    gamers = NULL;
}

/* Wham, game over. Called straight from drop_piece or msg.c */
void perform_game_over()
{
    int gx, fx, val, nummov;
    msg_movepieces *mptr = (msg_movepieces *)(&buf);

    /* return any held pieces to stash pads, just because */
    nummov = 0;

    for (gx=0; gx<numgamers; gx++) {
	fx = gamers[gx].holding;
	if (fx != (-1)) {
	    compute_piece(fx, gamers[gx].pad.x, gamers[gx].pad.y, pieces[fx].rot, 0);
	    emplace_piece(fx, gx);

	    mptr->m[nummov].piecenum = fx;
	    mptr->m[nummov].state = pieces[fx].state;
	    mptr->m[nummov].attack = pieces[fx].attack;
	    mptr->m[nummov].rot = pieces[fx].rot;
	    mptr->m[nummov].x = pieces[fx].x;
	    mptr->m[nummov].y = pieces[fx].y;
	    nummov++;
	}
    }
    if (nummov)
	for (gx=0; gx<numgamers; gx++) {
	    gesendmsg(gx, mptr, nummov*sizeof(struct msg_movepieces_sub));
	}

    /* compute scores */

    for (gx=0; gx<numgamers; gx++)
	gamers[gx].score = 0;

    for (fx=0; fx<numpieces; fx++)
	pieces[fx].score = 0;

    /* score defensive pieces */
    for (fx=0; fx<numpieces; fx++) {
	if (pieces[fx].state==STATE_GAME && !pieces[fx].attack) {
	    val = points_attacking_piece(fx, -1);	
	    if (val <= pieces[fx].size) {
		pieces[fx].score = pieces[fx].size;
		gamers[pieces[fx].gamernum].score += pieces[fx].size;
	    }
	}
    }
    /* score attacking pieces */
    for (fx=0; fx<numpieces; fx++) {
	if (pieces[fx].state==STATE_GAME && pieces[fx].attack) {
	    val = find_attacked_piece(fx);
	    if (val!=(-1) && pieces[val].score==0) {
		pieces[fx].score = pieces[fx].size;
		gamers[pieces[fx].gamernum].score += pieces[fx].size;
	    }
	}
    }

    /* zero out scores of icehoused players. This is done after the piece-scoring, because it will break otherwise. */
    for (gx=0; gx<numgamers; gx++)
	if (gamers[gx].icehoused)
	    gamers[gx].score = 0;

    final_board(); /* wipes piece and gamer structures, switches mode back */
}

